# SYSTEM ARCHITECTURE OF THE CE-RISE INFORMATION SYSTEM

**Deliverable 4.1**



| **Project:**             | Circular Economy Resource Information System                                                                                                             |
|--------------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------|
| **Acronym:**             | CE-RISE                                                                                                                                                  |
| **Grant Agreement:**     | 101092281                                                                                                                                                |
| **Funding Scheme:**      | Horizon Europe                                                                                                                                           |
| **Webpage:**             | www.ce-rise.eu                                                                                                                                           |
| **Work Package:**        | 4                                                                                                                                                        |
| **Work Package Leader:** | NILU                                                                                                                                                     |
| **Deliverable Title:**   | System architecture of the CE-RISE Information System                                                                                                    |
| **Deliverable Leader:**  | NILU                                                                                                                                                     |
| **Version:**             | 1.1                                                                                                                                                      |
| **Status:**              | Final                                                                                                                                                    |
| **Author(S):**           | Evert A. Bouman, Riccardo Boero, Donna Dykeman, Bendik Østrem Svalastog, Rune Åvar Ødegård, A.M. Esfar E Alam, Gaylord Booto, Miguel Las Heras Hernández |
| **E-Mail:**              | <eab@nilu.no>                                                                                                                                            |
| **Reviewed By:**         | Berend Mintjes, Joana F. Morgado, Cristina BB Guerreiro, Morten Pedersen, Stéphanie Muller, Golnoush Abbasi                                              |
| **Due Date:**            | 31 May 2024                                                                                                                                              |
| **Date Of Submission:**  | 30 June 2024 / revised 25 November 2024                                                                                                                  |
| **Dissemination Level:** | Public                                                                                                                                                   |

NOTICE

The contents of this document are the copyright of the CE-RISE
consortium and shall not be copied in whole, in part, or otherwise
reproduced (whether by photographic, reprographic or any other method),
and the contents thereof shall not be divulged to any other person or
organization without prior written permission. Such consent is hereby
automatically given to all members who have entered into the CE-RISE
Consortium Agreement, and to the European Commission to use and
disseminate this information.

This information and content of this report is the sole responsibility
of the CE-RISE consortium members and does not necessarily represent the
views expressed by the European Commission or its services. Whilst the
information contained in the documents and webpages of the project is
believed to be accurate, the author(s) or any other participant in the
CE-RISE consortium makes no warranty of any of any kind with regard to
this material.

# CONTENTS

- [EXECUTIVE SUMMARY](#executive-summary)

- [1 INTRODUCTION AND PURPOSE](#introduction-and-purpose)

  - [1.1 Circular Economy challenges and the CE-RISE Information System](#circular-economy-challenges-and-the-ce-rise-information-system)

  - [1.2 Objectives and purpose of the deliverable](#objectives-and-purpose-of-the-deliverable)

  - [1.3 The CE-RISE Information System Concept](#the-ce-rise-information-system-concept)

  - [1.4 Scope](#scope)

  - [1.5 Approach and structure](#approach-and-structure)

- [2 REQUIREMENTS](#requirements)

  - [2.1 Introduction](#introduction)

  - [2.2 Requirements Overview](#requirements-overview)

    - [2.2.1 Requirement 1: Secure Access and Data Privacy
](#requirement-1-secure-access-and-data-privacy)

    - [2.2.2 Requirement 2: Effective Integration and Extension of External
Information
](#requirement-2-effective-integration-and-extension-of-external-information)

    - [2.2.3 Requirement 3: Computation and Provision of Novel Added-Value
Information
](#requirement-3-computation-and-provision-of-novel-added-value-information)

  - [2.3 End-User Requirements Capture](#end-user-requirements-capture)

  - [2.4 Stakeholders to the System Architecture](#stakeholders-to-the-system-architecture)
  
  - [2.5 Quality Goals](#quality-goals)

- [3 ARCHITECTURE CONSTRAINTS](#architecture-constraints)

  - [3.1 Evolving DPP Legal Status and Standards](#evolving-dpp-legal-status-and-standards)

    - [3.1.1 DPP Provider Agnosticism](#dpp-provider-agnosticism)

  - [3.2 Compliance with Data Protection Legislation](#compliance-with-data-protection-legislation)

  - [3.3 Budget Constraints for Autonomous Key Partners](#budget-constraints-for-autonomous-key-partners)

  - [3.4 Intellectual Property Management](#intellectual-property-management)

  - [3.5 Availability of Technology Stack](#availability-of-technology-stack)

- [4 SYSTEM SCOPE AND CONTEXT](#system-scope-and-context)

  - [4.1 Business Context](#business-context)

    - [4.1.1 Software Systems Interacting with the Infrastructure](#software-systems-interacting-with-the-infrastructure)

    - [4.1.2 CE-RISE initial dataset and model providers](#ce-rise-initial-dataset-and-model-providers)

    - [4.1.3 User personas](#user-personas)

    - [4.1.4 Other Software Systems Used by CE-RISE IS Users](#other-software-systems-used-by-ce-rise-is-users)

    - [4.1.5 Business Context Summary](#business-context-summary)

  - [4.2 Technical Context](#technical-context)

    - [4.2.1 DPP via REST API](#dpp-via-rest-api)

    - [4.2.2 User Authentication](#user-authentication)

    - [4.2.3 User Data Upload](#user-data-upload)

    - [4.2.4 Other Data Ingestion through REST API](#other-data-ingestion-through-rest-api)

    - [4.2.5 Internal API Gateway](#internal-api-gateway)

- [5 SOLUTION STRATEGY](#solution-strategy)

  - [5.1 Solution Strategy Concepts](#solution-strategy-concepts)

  - [5.2 Technology Stack](#technology-stack)

  - [5.3 Version Management and Release Strategy](#version-management-and-release-strategy)

- [6 BUILDING BLOCK VIEW](#building-block-view)

  - [6.1 C4 – Container View](#c4-container-view)

    - [6.1.1 DPP I/O (black box)](#dpp-io-black-box)

    - [6.1.2 RE Service (black box)](#re-service-black-box)

    - [6.1.3 SEE Service (black box)](#see-service-black-box)

    - [6.1.4 Product Information (black box)](#product-information-black-box)

    - [6.1.5 Semantic Search (black box)](#semantic-search-black-box)

    - [6.1.6 Databases (black box)](#databases-black-box)

    - [6.1.7 Data Ingestion Service (black box)](#data-ingestion-service-black-box)

    - [6.1.8 Web-App/Front-End (black box)](#web-appfront-end-black-box)

    - [6.1.9 API Gateway (black box)](#api-gateway-black-box)

    - [6.1.10 Authentication and Authorization (black box)](#authentication-and-authorization-black-box)

  - [6.2 C4-Component Diagrams](#c4-component-diagrams)

    - [6.2.1 DPP I/O (white box)](#dpp-io-white-box)

    - [6.2.2 RE-Service (white box)](#re-service-white-box)

    - [6.2.3 SEE Service (white box)](#see-service-white-box)

    - [6.2.4 Product Information (white box)
62](#product-information-white-box)

    - [6.2.5 Semantic Search (white box)](#semantic-search-white-box)

    - [6.2.6 Databases Module (white box)](#databases-module-white-box)

    - [6.2.7 Other Data Ingestion Routines (white box)](#other-data-ingestion-routines-white-box)

    - [6.2.8 Web-App/Front-End (white box)](#web-appfront-end-white-box)

    - [6.2.9 API Gateway (white box)](#api-gateway-white-box)

- [7 RUNTIME VIEW](#runtime-view)

  - [7.1 Retrieve Information from a DPP](#retrieve-information-from-a-dpp)

  - [7.2 Calculate RE-indicators for a Product](#calculate-re-indicators-for-a-product)

  - [7.3 Calculate SEE-impact for a Product](#calculate-see-impact-for-a-product)

  - [7.4 Comparative SEE Impacts for RE Strategies](#comparative-see-impacts-for-re-strategies)

  - [7.5 Roll-Up of All Processed Products by Single Actor](#roll-up-of-all-processed-products-by-single-actor)

  - [7.6 Semantic Search Workflow](#semantic-search-workflow)

  - [7.7 Traceability of Secondary Raw Materials](#traceability-of-secondary-raw-materials)

  - [7.8 Consumer Views CE Information](#consumer-views-ce-information)

  - [7.9 Write to DPP Through Information System](#write-to-dpp-through-information-system)

- [8 DEPLOYMENT VIEW](#deployment-view)

  - [8.1 Deployment During the CE-RISE Project](#deployment-during-the-ce-rise-project)

  - [8.2 Long Run Production System](#long-run-production-system)

- [9 CROSS-CUTTING CONCEPTS](#cross-cutting-concepts)

  - [9.1 Communication and Integration](#communication-and-integration)

  - [9.2 Session Handling, Logging and Tracing](#session-handling-logging-and-tracing)

  - [9.3 Containerization of Building Blocks](#containerization-of-building-blocks)

  - [9.4 Code Documentation](#code-documentation)

  - [9.5 UI, UX, and Accessibility](#ui-ux-and-accessibility)

  - [9.6 Version Control and Branch Management](#version-control-and-branch-management)

- [10 ARCHITECTURAL DECISIONS](#architectural-decisions)

  - [10.1 Main Architectural Decisions](#main-architectural-decisions)

    - [10.1.1 Microservice Architecture](#microservice-architecture)

    - [10.1.2 Use of Available Resources](#use-of-available-resources)

  - [10.2 Approaches Not Used for the CE-RISE Information System](#approaches-not-used-for-the-ce-rise-information-system)

    - [10.2.1 Container Orchestration](#container-orchestration)

    - [10.2.2 Deployment on Commercial Cloud Environments
](#deployment-on-commercial-cloud-environments)

- [11 QUALITY REQUIREMENTS](#quality-requirements)

  - [11.1 Quality Tree](#quality-tree)

  - [11.2 Quality Scenarios](#quality-scenarios)

- [12 TECHNICAL RISKS](#technical-risks)

- [13 CONCLUDING REMARKS](#concluding-remarks)

- [GLOSSARY](#glossary)

- [REFERENCES](#references)



# EXECUTIVE SUMMARY 

The CE-RISE Information System (IS) is an ambitious initiative designed
to accelerate the EU manufacturing industry’s capacity to adopt the
principles of the Green Deal. Its primary aim is to facilitate the
implementation of RE-strategies (reuse, repair, repurpose, refurbish,
remanufacture, recycle) by providing value chain actors with the
necessary information to make informed decisions regarding post-use
product treatment, track product service life, and demonstrate
environmental benefits. The system is conceptually described in CE-RISE
project deliverable D2.1 and this system architecture document builds on
this to meet requirements from various stakeholders, including
government, industry, researchers, digitalization leaders, and
consumers.

**Key Challenges and Solutions**

Several challenges to adopting circular economy practices for electrical
and electronic equipment (EEE) and batteries are identified, categorized
into systemic/legal, technical, economic, and social/behavioral aspects.
Key issues include intellectual property rights, lack of repair
availability, high costs, and consumer behavior. The CE-RISE information
system leverages Digital Product Passports (DPPs) to address these
challenges, providing traceability and information sharing across the
value chain. The system can improve the management of secondary
materials and support informed decision-making for repair and other
RE-strategies.

**Objectives and Purpose of the Deliverable**

This document details the IT architecture of the CE-RISE information
system, serving as a guideline for developers and providing various
stakeholders with a comprehensive overview of the system’s functionality
and design. The architecture supports secure access and data privacy,
effective integration of external information, and advanced data
processing to generate novel added-value information. The development
process, which spans until December 2026, will continually update the
system based on inputs from industry partners and scientific
collaborators.

**The CE-RISE Information System Concept**

The CE-RISE information system aims to facilitate information sharing
among value chain actors to enhance waste reduction, improve the use of
secondary raw materials, and assess environmental and socio-economic
impacts of products. The system's functionality is based on three main
concepts:

-   **RE-criteria**: Threshold levels for reusability, repairability,
    repurposability, refurbishability, remanufacturability, and
    recyclability.

-   **Digital Product Passport (DPP)**: A repository for product
    information accessible by downstream stakeholders.

-   **Impact Calculations**: Assessing socio-economic and environmental
    (SEE) impacts through advanced data processing.

The system architecture includes a RE-module, SEE module, and Product
Information module, supported by a front-end interface and data storage
layer. It interacts with external data sources and service providers to
ensure comprehensive value chain information.

**Quality Goals and Architecture Constraints**

The architecture emphasizes security, performance, reliability,
usability, scalability, maintainability, data quality, and compliance
with relevant standards and regulations. It must also accommodate
evolving legal standards for DPPs, comply with data protection
legislation, manage budget constraints, handle intellectual property,
and utilize the available technology stack effectively.

**Business Context**

The CE-RISE information system is designed to integrate with various
software systems, including Digital Product Passport (DPP) services and
supply chain data management tools. Key elements include:

-   DPP Services: Represented by Circularise, providing data for
    supported products. Users can write CE-RISE results to their DPP.

-   Initial Dataset Providers: The system incorporates data from
    external sources and project-developed datasets into the CE-RISE
    databases.

-   Life Cycle Inventory (LCI) Data: Essential for enabling the SEE
    module to compute environmental impact indicators.

-   Environmentally Extended Input Output (EEIO) Tables: These tables
    provide valuable information for environmental impact assessments.

In technical terms, the system is intended to communicate internally
through REST APIs, enabling seamless communication and data exchange.
Users interact with the system through a dedicated front-end web
application. This system architecture describes a modular approach,
ensuring scalability and flexibility through containerization and
well-defined microservices.

**Key Modules and Services**

The system is composed of several key modules and services described as
fundamental building blocks in the system architecture. These are:

-   RE Service: Calculates RE-indicators and circularity metrics.

-   SEE Service: Computes socio-environmental-economic impact
    indicators.

-   Product Information Service: Supports decision-making and storage of
    product specific results.

Support infrastructure, such as search functionality and dedicated
services and databases for data ingestion and data storage are also
identified.

**Sequence Diagrams**

The interplay between the microservices and databases comprising the
Information System is documented in the form of sequence diagrams. For
example, users can retrieve product information, such as assembly
manuals, repairability indices, and SEE (Sustainability, Environmental,
and Economic) impacts by scanning QR codes. This involves coordinated
interactions between the front-end application, API Gateway, and various
services like DPP I/O, Product Information, RE, and SEE services.


**Deployment**

In the course of the CE-RISE project, the system will be developed and
deployed on servers provided by NILU, making use of scalable virtual
machines as well as a dedicated development and production environment.

**Conclusion**

The CE-RISE Information System is poised to significantly advance the
circular economy within the EU manufacturing sector by providing a
robust digital infrastructure for information sharing and
decision-making. By addressing key challenges and adhering to stringent
requirements and quality goals, the CE-RISE IS aims to enhance
sustainability and circularity in value chains, contributing to the
broader objectives of the Green Deal.

# INTRODUCTION AND PURPOSE

## Circular Economy challenges and the CE-RISE Information System 

The CE-RISE Information System (IS) is a new ambition to address the
need to accelerate the capacity of the EU manufacturing industry to
adopt the principles of the Green Deal and supporting regulations for
producer responsibility and move towards more circular value chains.

The main aim of the CE-RISE information system is to enable the
successful implementation of RE-strategies[1]. The information system is
envisioned to provide an information basis to enable value chain actors
to make decisions for post-use treatment of products (e.g. to repair or
remanufacture), track product service life and decisions made by
downstream actors once a product is subjected to a RE-strategy and
enable value chain actors to demonstrate the combined effects (e.g.
emissions saved) associated with making informed choices enhancing
circularity.

The information system is conceptually described in CE-RISE project
deliverable D2.1 and briefly summarized in Chapter in Section 1.3. This
architecture describes the technical design of the digital
infrastructure comprising the information system. It builds on the
information system concept and aims to include requirements from both
CE-RISE research partners and identified End-users.

Within the framework of European projects (reviewed in CE-RISE project
deliverable D1.1), the CE-RISE project takes a step beyond the need to
establish and deploy the DPP to focus on using the information in the
DPP for decision-making. However, there are still many challenges for
industry to digitalize their existing knowledge and information for
decision-making. Table 1 outlines challenges foreseen as described by
the European Topic Centre on Circular Economy (ETC/CE et al., 2022)
modified and adapted with inputs from CE-RISE stakeholders and end-users
(as collected in the conversations about information system User
Stories, see Section 2.3).

<span id="_Ref170487770" class="anchor"></span>Table 1: Key challenges
to Circular strategies for Electrical and Electronic Equipment (EEE) and
batteries, with elements highlighted dependent on information.

<table>
<thead>
<tr class="header">
<th><strong>Systemic/Legal</strong></th>
<th><strong>Technical</strong></th>
<th><strong>Economic</strong></th>
<th><strong>Social and Behavior</strong></th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td><ul>
<li><p>Intellectual property rights</p></li>
<li><p>Lack of availability or access to repair</p></li>
<li><p><em><strong>Lack of access to spare parts and technical information and training</strong></em></p></li>
<li><p>Lack of standardization and interoperability of key product components</p></li>
<li><p>New eco-design rules have limited application</p></li>
</ul>
<p>Right to Repair:</p>
<ul>
<li><p>New eco-design rules only apply to commercial repair</p></li>
<li><p>Liability issues</p></li>
<li><p>Safety concerns and interference with OEMs warranty (if still applicable)</p></li>
</ul></td>
<td><ul>
<li><p>Some product designs do not allow for easy repair and/or replacement of broken parts (complex, non-modular)</p></li>
<li><p><em><strong>Increased technical knowledge required due to growing complexity of products and increased incorporation of electronic components and component miniaturization</strong></em></p></li>
<li><p>Software issues can interfere with success of a physical repair</p></li>
<li><p>Short innovation cycle</p></li>
<li><p><em><strong>Difficult in providing repair services to a large range of products and brands</strong></em></p></li>
<li><p><em><strong>Decreasing number of experts with technical knowledge and skills</strong></em></p></li>
<li><p>Lack of standard quality guidelines for reuse of materials</p></li>
<li><p><em><strong>Bills of Materials (BoM) are incomplete and not sufficiently reliable (upwards of 80% of the BoM)</strong></em></p></li>
<li><p><em><strong>Lack of knowledge of BoMs and designs for bought-in parts</strong></em></p></li>
<li><p><em><strong>Updated Restricted Substances lists which influence multiple product life cycles</strong></em></p></li>
<li><p><em><strong>‘How to’ knowledge for repair is not centralized, digitalized</strong></em></p></li>
</ul></td>
<td><ul>
<li><p>High costs for spare parts and logistics</p></li>
<li><p>High costs for labor compared to buying new components or products</p></li>
<li><p><em><strong>Low RoI apart from Critical and Strategic Raw Material (CRM) Capture</strong></em></p></li>
<li><p>Growing share of online trade increases the complexity of the value chain</p></li>
<li><p>Just-in-time manufacturing leads to poor supply chain resiliency</p></li>
<li><p>Secondary Raw Material (SRM) supplies are currently not reaching demand scales for wide adoption and resiliency (JRC, 2024)</p></li>
<li><p>Lack of symbiotic transition between end-of-life and waste-as-a-resource</p></li>
</ul></td>
<td><p>Consumer:</p>
<ul>
<li><p><em><strong>Lack of skills and knowledge</strong></em></p></li>
<li><p>Lack of emotional attachment</p></li>
<li><p>Attraction of new products sometimes offering new features</p></li>
</ul>
<p>Businesses:</p>
<ul>
<li><p>Decreasing demand for repair services</p></li>
<li><p>Low customer expectations of durability and repairability of products</p></li>
<li><p>Willingness to maintain products beyond a limited life span</p></li>
</ul></td>
</tr>
</tbody>
</table>

Source: Adapted from (ETC/CE et al., 2022)

Table 1 highlights challenges to adopting Circular Economy practice,
broken down in systemic/legal, technical, economic, and social
behavioral aspects. In addition, we highlight which challenges can be
ameliorated by access to detailed information related to the product,
its components and its materials. The latter point can be directly
addressed through the use of Digital Product Passports (DPPs). By
creating a digital ledger that is accessible by actors across the value
chain (Vlachos and Damvakeraki, 2024), DPPs enable material
traceability, important for keeping track of the availability of high
value added secondary materials, as well as provide an infrastructure
for storing and transmitting information enabling activities such as
repair.

Linked DPPs can further improve this by enabling the transmission of
relevant product and process information between economic actors
transcending the direct purchase-supply transaction relation. This
functionality is for example of particular importance in tracking
secondary materials embedded in products as product manufacturers
(original equipment manufacturers – OEM) rarely use raw materials
directly. Raw material supply is therefore located a few tiers upstream
in the value chain. Linked DPPs allow parties to request information
from any upstream activities, regardless of location in the value chain

However, at present, very few products and components have associated
digital product passports. In addition, linked-DPPs as information
carriers require processing software to extract complex information
elements such as a calculated secondary raw material content of a
product.

The CE-RISE information system leverages the potential power of the
DPPs, has access to external data sources that can fill in missing
elements in the value chain mapping, and can calculate relevant
indicators for both Circular Economy as well as for Sustainability.

## Objectives and purpose of the deliverable

This deliverable describes the IT architecture that implements the
CE-RISE information system. The primary goal of this document is to
present the technical design concepts of the CE-RISE information system.
The architecture documentation acts as a guideline for the CE-RISE
information system developer team and outlines the expectation towards
the system in terms of functional and non-functional requirements. The
architecture also serves as a documentation for the anticipated
functionality of the CE-RISE information system, and it presents
different views of the system, and its building blocks and underlying
information flows. By presenting different views of the system
architecture, this documentation enables various non-developer
stakeholders within CE-RISE (such as anticipated users for the case
studies in Work Package 5, scientific partners, and external advisors)
to evaluate the software architecture from their respective perspective.

*“Good architecture makes the system easy to understand, easy to
develop, easy to maintain, and easy to deploy”* (Martin and Martin,
2018)*.*

The development of the CE-RISE information system is expected to
continue for 2.5 years until December 2026. This document can be
considered a first version of the system documentation and is made
available to the public. To keep track of changes and to provide an
up-to-date overview of the system components, an internal version of
this report is made available under version management here
<https://git.nilu.no/ce-rise/architecture> and updated throughout the
course of the project as the codebase grows.

## The CE-RISE Information System Concept

The CE-RISE Information system is a digital infrastructure aimed at
facilitating the information sharing among actors across value chain to
enable increased waste reduction, improved use and tracing of secondary
raw materials in product life cycles, and access to compute
environmental and socio-economic impacts of products and components.
Access to this information is expected to enable users to gain insights
into the secondary raw material content of singular products and allow
them to structure their business processes more accurately towards
circular and sustainable practices.

The functionality of the CE-RISE information system hinges upon three
main concepts and technologies:

-   The definition of RE-criteria – a set of criteria defining threshold
    levels for the RE-indicators reusability, repairability,
    repurposability, refurbishability, remanufacturability, and
    recyclability of product and their embedded components.

-   The utilization of a digital product passport service as a product
    information repository and allowing downstream stakeholders to
    access relevant primary product or component data as supplied by the
    original value chain actor.

-   The capability of calculating (secondary) material overviews,
    socio-economic and environmental product life cycle impacts (SEE
    impacts). This includes impact calculation for different downstream
    processing scenarios such that RE-impacts (SEE impacts pertaining to
    each RE-strategy) can be calculated.

Figure 1 showcases a high-level depiction of the CE-RISE information
system, its interplay with external entities such as a DPP service
provider, and external data supply to the system. The concept builds on
the assumption that DPP owners create DPPs for their outgoing products,
which can be raw material batches, components, final products from
original equipment manufacturers (OEMs), as well as products, components
or materials which have been subject to a RE-activity, i.e. they have
been repaired, refurbished, remanufactured, reused, repurposed, or
recycled. Each actor uses input data from their own data sets and
software systems to populate the DPP, the service for which is offered
by the DPP service provider Circularise. Once information is stored in
the DPP, this information will become available to downstream actors,
e.g. a collector can read out relevant information supplied to the DPP
by the OEM. Users can subsequently use the CE-RISE information system
depicted in Figure 1 to access and calculate product information which
requires a systemic understanding of the value chain and its operations,
e.g. the calculation of secondary raw materials, environmental impacts,
or the read-out of product specific information such as assembly manuals
or the availability of takeback schemes.

The information system concept in Figure 1 highlights three main data
processing modules, the RE-module, the SEE module and the Product
information module, in addition to a front-end interface for user
interaction and a data storage support layer. The information system can
uptake information from external sources and have two-way communication
with service providers such as offered through the CE-RISE project
partner Circularise.

<img src="media/fig1.png" style="width:6.39444in;height:3.60351in" alt="The CE-RISE information system" />

<span id="_Ref170488871" class="anchor"></span>Figure 1: Overview of the
CE-RISE conceptual framework (CE-RISE deliverable 2.1)

## Scope

This deliverable concerns the initial design of the software structure
making up the CE-RISE Information System. The Information System enables
linking information available in DPPs through DPP service providers with
other data sources and subsequent making this information available
through a dedicated front-end web app. It presents a blueprint of the
architecture of the three CE-RISE information system modules as
envisioned in Figure 1 as well as the supporting digital infrastructure
required for delivering the functionality of these modules, such as
databases and user authentication and authorization.

The CE-RISE information system operates in a context and is dependent on
various external information sources. One example here is the software
operated by the DPP service provider Circularise. The information system
is designed to interact with these external software systems through
application programming interface (API) and for the purpose of this
deliverable we do not focus on the internal technical architecture of
external systems, but rather specify the link and treat them further as
black boxes acting as information source or reception point.

## Approach and structure 

The system architecture follows from the work of Work Package 4 in
concert with Work Packages 2, 3 and 5. Based on the project description
of action a conceptual taskforce operating across work packages was
created that discussed the CE-RISE information system concepts. The
industrial case study partners from work package 5 were asked to provide
inputs on their envisioned user stories and use cases of the CE-RISE
information system – documenting the expected functionality of the
system and its inclusion in anticipated workflows. All 13 industry
partners contributed more than 130 User Stories representing 9 Primary
Personas which articulated jobs for the CE-RISE IS and related modules.
User Stories and related functionality were prioritized using the MoSCoW
method[2] by industry members, followed by mapping to Use Cases and
Features by software solution owners and contributing researchers. In
line with common software practice, the implementation team in Work
Package 4 made key architectural decisions (as documented in Section
10.1) – primarily that the system will be set-up as a distributed system
of micro-services, thus allowing to gradually update the systems
functionalities in the course of the project, while not hampering the
use and testing of other functionalities in the case studies.

This document follows where possible the open-source
[arc42](https://arc42.org)[3] template for the documentation of software
and system architecture (Hruschka and Starke, 2023). This template was
chosen for its structured approach enabling different viewpoints to the
architecture. The scope of the arc42 template is extended and
supplemented with elements specific to the CE-RISE project,
organizational structure as a Horizon Europe project.[4]

The following Chapters describe the system requirements and quality
goals (Chapter 2), the constraints for implementation of the
architecture (Chapter3 ). The system scope and context (Chapter4),
solution strategies (Chapter 5), building block view of the individual
elements of the CE-RISE information system (Chapter 6), runtime view
documenting various generic use cases (Chapter 7), consideration for the
initial deployment of the system (Chapter 8), Cross-cutting concepts
(Chapter 9) describing the relation to the a log of architectural
decisions and overview of expected maintenance (Chapter 11). Chapter 11
provides a more extensive overview of quality requirements and
scenarios. Chapter 12 discusses technical risks and, finally, concluding
remarks are presented in Chapter 13.

# REQUIREMENTS

## Introduction

This software architecture documentation addresses various functional
and non-functional requirements, guiding the team in understanding
constraints, testing requirements, following overarching concepts, and
considering specific architectural decisions during feature
implementation. The documentation plays a role in guiding team members,
especially ones that are added in later stages of the project, by
providing answers to questions related to the system's building blocks,
technology choices, deployment methods, and relevant conventions.
Transparent and understandable architectural decisions are emphasized,
ensuring that team members are aware of the reasoning behind the choices
made, thereby supporting a collaborative and informed team environment.
The documentation also acts as a reference for external stakeholders,
offering stakeholder-appropriate views of the software architecture and
facilitating effective communication between the development team and
external partners.

The system’s main objective is to provide users with added value
information about products’ properties, circularity’ and social,
environmental and economic impacts, in order to aid sustainable and
circular economy decision making. The information provided is aimed to
be generated following state-of-the-art scientific approaches, with high
standards of data confidentiality and protection throughout data
pipelines. As outlined in the concept Figure 1, the system calls for
multiple data sources and intricate data transformations. Consequently,
the system architecture includes a listing of various input interfaces
and example data pipelines.

Section 2.2 described the Requirements overview to the ***system
architecture*** from an abstract perspective, starting from 3 main
requirements. The architecture should be capable of supporting secure
access and data privacy, allow for effective integration of external
information sources, and provide an infrastructure suited for advanced
data processing and computation required to generate novel added-value
information.

While section 2.2 provides the basis for the architecture, section 2.3
summarizes the requirements posed upon the ***Information system*** by
its intended end-users. In this section, we map the user stories to
generalized expected functionalities. In section 2.4 we list the
identified stakeholders to the system architecture, and we conclude this
Chapter in section 2.5 with an elaboration of the quality goals of the
architecture.

## Requirements Overview

The system has 3 main requirements summarized in the following list
along with their associated functional and non-functional requirements.
Overall, there are 14 functional requirements and 6 non-functional
requirements.

<span id="_Toc170654496" class="anchor"></span>Table 2: Requirement to
the system architecture

<table>
<thead>
<tr class="header">
<th><strong>Main Requirement</strong></th>
<th><strong>Functional Requirements</strong></th>
<th><strong>Non-Functional Requirements</strong></th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td>Secure Access and Data Privacy</td>
<td><ul>
<li><blockquote>
<p>Authentication and Authorization</p>
</blockquote></li>
<li><blockquote>
<p>Encryption</p>
</blockquote></li>
<li><blockquote>
<p>Privacy Compliance</p>
</blockquote></li>
<li><blockquote>
<p>Audit Trail</p>
</blockquote></li>
<li><blockquote>
<p>Intrusion Detection and Prevention</p>
</blockquote></li>
</ul>
<p>Secure Communication</p></td>
<td><ul>
<li><blockquote>
<p>Performance</p>
</blockquote></li>
<li><blockquote>
<p>Scalability</p>
</blockquote></li>
</ul>
<p>Usability</p></td>
</tr>
<tr class="even">
<td>Effective Integration and Extension of External Information sources</td>
<td><ul>
<li><blockquote>
<p>Data Integration</p>
</blockquote></li>
<li><blockquote>
<p>Data Quality Enforcement, Transformation, and Enrichment</p>
</blockquote></li>
<li><blockquote>
<p>API and Protocol Support</p>
</blockquote></li>
</ul>
<p>Scalability and Extensibility</p></td>
<td><ul>
<li><blockquote>
<p>Performance</p>
</blockquote></li>
<li><blockquote>
<p>Reliability and Availability</p>
</blockquote></li>
</ul>
<p>Data Quality</p></td>
</tr>
<tr class="odd">
<td>Computation and Provision of Novel Added-Value Information</td>
<td><ul>
<li><blockquote>
<p>Advanced Data Processing</p>
</blockquote></li>
<li><blockquote>
<p>Data Pipeline Certification</p>
</blockquote></li>
<li><blockquote>
<p>Integration of Scientific Models</p>
</blockquote></li>
</ul>
<p>Data Aggregation and Presentation</p></td>
<td><ul>
<li><blockquote>
<p>Performance</p>
</blockquote></li>
<li><blockquote>
<p>Scalability</p>
</blockquote></li>
</ul>
<p>Quality Assurance</p></td>
</tr>
</tbody>
</table>

### Requirement 1: Secure Access and Data Privacy

#### Objective

The software system must provide a robust and secure access mechanism to
safeguard both the system and its associated data sources. This includes
ensuring data privacy and fulfilling all authentication requirements.

#### Functional Requirements

**Authentication and Authorization:** The system shall implement a
multi-factor authentication (MFA) mechanism for user authentication.
User roles and permissions shall be defined, allowing access control to
specific functionalities and data based on user roles. The system will
refrain from storing or utilizing users' credentials for accessing
external data providers. Instead, it will employ an authorization token
system to ensure secure user identification and data retrieval from
external data sources effectively.

**Encryption:** All data transmitted over the network shall be encrypted
using industry-standard encryption protocols. Sensitive data stored
within the system will only concern user identification and
authorization and they shall be encrypted at rest to prevent
unauthorized access.

**Privacy Compliance:** The system shall comply with relevant data
protection regulations and standards, such as GDPR (General Data
Protection Regulation) in the European Union, to ensure the privacy of
user data.

**Audit Trail:** A comprehensive audit trail shall be maintained,
recording all access and modification activities within the system for
auditing and forensic purposes.

**Intrusion Detection and Prevention:** The system shall incorporate
intrusion detection and prevention mechanisms to proactively identify
and mitigate security threats.

**Secure Communication:** Communication between system components and
external interfaces shall use secure protocols and certificates to
prevent eavesdropping and man-in-the-middle attacks.

#### Non-Functional Requirements

**Performance:** The authentication and access control processes shall
have minimal impact on system performance to ensure smooth user
experiences.

**Scalability:** The security infrastructure shall be scalable to
accommodate future growth in user base and data volume.

**Usability:** User interfaces for authentication and access control
shall be user-friendly to facilitate efficient and error-free
interactions.

#### Compliance

The software architecture shall adhere to industry best practices and
standards for secure system design, such as OWASP (Open Web Application
Security Project) guidelines and ISO 27001 for information security
management.

### Requirement 2: Effective Integration and Extension of External Information

#### Objective

The software system must seamlessly integrate and extend diverse,
sparse, and dispersed information and knowledge from external systems to
enrich its functionality and data sources.

#### Functional Requirements

**Data Integration:** The system shall provide connectors and interfaces
to external data sources, enabling the retrieval and integration of
diverse data types and formats. It shall support real-time data
ingestion and batch processing as needed to accommodate various data
sources.

**Data Quality Enforcement, Transformation, and Enrichment:** The system
shall include data transformation capabilities to convert data from
external sources into a common format for internal use. Data quality
will also be checked and enforced through API specifications and data
transformation procedures. Data enrichment processes shall be defined to
augment incoming data with relevant metadata or contextual information.

**API and Protocol Support:** Due to the straightforward approach and
scalability of REST (Representational State Transfer), the system will
be based on this protocol for handling data through HTTPS requests.
Should the need arise, the system will also implement SOAP or GraphQL.

**Scalability and Extensibility:** The architecture shall be designed to
accommodate the addition of new external data sources with minimal
disruption, ensuring scalability and extensibility.

#### Non-Functional Requirements

**Performance:** Data integration and extension processes shall be
optimized for performance to minimize latency and provide real-time or
near-real-time updates when required.

**Reliability and Availability:** The system shall ensure high
availability and reliability for data integration, with failover
mechanisms and redundancy to prevent data loss.

**Data Quality:** Measures shall be in place to ensure the quality,
accuracy, and consistency of integrated data, including data cleansing
and validation procedures.

#### Compliance

The software architecture shall align with industry standards for data
integration, such as those defined by the Data Integration and ETL
(Extract, Transform, Load) best practices and the Enterprise Integration
Patterns (EIP) framework.

### Requirement 3: Computation and Provision of Novel Added-Value Information

#### Objective

The software system must employ state-of-the-art scientific approaches
and certified data pipelines and integrations to compute and provide
novel added-value information to system users, enhancing the utility and
insight derived from incoming data.

#### Functional Requirements

**Advanced Data Processing:** The system shall incorporate advanced data
processing algorithms, including machine learning, artificial
intelligence, statistical analysis, and Environmentally Extended Input
Output (EE-IO) and Life Cycle Assessment (LCA) approaches to derive
novel insights from incoming information.

**Data Pipeline Certification:** Certified data pipelines shall be
established to ensure the reliability and integrity of incoming data.
Data from external sources shall undergo validation and quality checks
and required transformations before being used in computations.

**Integration of Scientific Models:** The system shall integrate
certified scientific models and frameworks relevant to the domain to
enhance the accuracy of computed information. Scientific models shall be
regularly reviewed and updated to reflect the latest research and
knowledge.

**Data Aggregation and Presentation:** Computed information shall be
aggregated and presented to system users in a clear and user-friendly
manner, facilitating decision-making. Visualization techniques and data
presentation best practices shall be applied.

#### Non-Functional Requirements

**Performance:** Computation processes shall be optimized for
performance, ensuring that results are delivered in a timely manner to
meet user expectations.

**Scalability:** The architecture shall be designed to scale
horizontally and vertically to accommodate increased computational
demands as the user base and data volume grow.

**Quality Assurance:** Quality assurance processes shall be implemented
to validate the correctness and reliability of computed information.

#### Compliance

The software architecture shall adhere to domain-specific practices for
scientific data processing and integration, such as those applied by
organizations like the International Society for Industrial Ecology
(ISIE) and the National Institute of Standards and Technology (NIST).

## End-User Requirements Capture

As part of a combined activity between Tasks 4.1 and 5.1 in the CE-RISE
project, specific User Stories for the CE-RISE Information System were
assembled with the support of industry input. Together with input from
the scientific partners of the CE-RISE consortium, the User Stories
formed the basis of the envisioned feature of the CE-RISE information
system, and 9 distinct runtime scenarios specified in Chapter 7. To
validate that these scenarios cover most of the User Stories, we include
here an overview mapping of the scenarios against a summary of user
stories. The User Stories added to Table 3 below are listed regardless
of their prioritization by the End-User and Solution Owners.

<span id="_Ref169870399" class="anchor"></span>Table 3: Mapping of
generalized expected functionalities of the system by the user stories

| **Expected functionalities of the Information System**                                                                                                                                                                               | **Section** | **User story**                                                                                                                                                                                                                                                                                            |
|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|-------------|-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Retrieve information from a DPP                                                                                                                                                                                                      | 7.1         | 30% of stories (possibly more) indicated the need to gather information from the DPP for reporting/declaration (e.g. substances, secondary raw materials content, CRM, BoMs, SEE-impacts), calculations (e.g. RE-indicators, SEE-indicators), instructions for RE-strategies, benchmarking products, etc. |
| Calculate RE-indicators for a product                                                                                                                                                                                                | 7.2         | Direct mention in 13 user stories representative of both OEM and EOL value chain actors.                                                                                                                                                                                                                  |
| Calculate SEE-impact for a RE-Strategy (includes recycled, reused, CRM, restricted substances material and component content)                                                                                                        | 7.3         | Direct mention in 24 user stories representative of raw materials production, OEMs, and EOL value chain actors.                                                                                                                                                                                           |
| Calculate a RE-Strategy (theoretical and actual)                                                                                                                                                                                     | 7.4         | Direct mention in 25 user stories representative of OEM and EOL value chain actors.                                                                                                                                                                                                                       |
| Benchmark SEE impact for a RE-Strategy                                                                                                                                                                                               | 7.4         | The benchmarking of SEE impacts in feedstocks as well as refurbished products was mentioned in 8 user stories representative of materials production and OEM value chain actors.                                                                                                                          |
| Compliance confirmation of an SEE impact for a RE-Strategy                                                                                                                                                                           |             | Direct mention in 21 user stories representative of raw materials production, OEM, and EOL value chain actors.                                                                                                                                                                                            |
| Roll-up of SEE-impact of other KPI of interest                                                                                                                                                                                       | 7.5         | Direct mention by 17 user stories, to document cumulative effects of SEE impact reduction over a (processed) product portfolio in a given time.                                                                                                                                                           |
| Semantic search workflow                                                                                                                                                                                                             | 7.6         | Mentioned by 18 user stories focusing on search for relevant links, WEEE parts, OEM instructions, BoM information and environmental impact (LCA) related information.                                                                                                                                     |
| Traceability of secondary raw materials                                                                                                                                                                                              | 7.7         | Direct mention in 8 user stories by raw materials production and OEM value chain actors.                                                                                                                                                                                                                  |
| Traceability of Critical and Strategic Raw Materials                                                                                                                                                                                 | 7.7         | Direct mention in 6 user stories.                                                                                                                                                                                                                                                                         |
| Consumer view Circular Economy information (industry partner projecting their interest to Write information to the DPP for Consumers, or any downstream actor for HSE, declarations, etc.; no comment on public/private information) | 7.8         | The ability to communicate (public) CE related information to consumers or downstream actors was highlighted in 29 user stories representative of both upstream and downstream actors along the value chain.                                                                                              |
| Write information to DPP through CE-RISE information system                                                                                                                                                                          | 7.9         | To be confirmed – could relate to stories for downstream actors as above; also interest to have a DDP and calculations readiness assessment against product category and/or traceability requirements                                                                                                     |

There are many more User Stories shared by End-Users around the topic of
Automation and Workflow Management which relate to the Challenges in
Table 1. However, these aspects are not specifically within the scope of
the project but will be explored due to the high priority given by the
project partners and the presence of two software houses as partners.
Three categories of Automation and Workflow Management are evident in
these stories:

1.  In-House data/information management for DPP population: A
    well-structured template may be easily uploaded to a data management
    system to perform further analytics, however, populating the
    template may take considerable time and effort by the End-User.
    In-House software tools (owned or licensed by the End-User) for
    engineering workflows often offer various levels of interoperability
    and integration needed for decision-making across multiple
    stakeholders (or Personas) in the business. We will explore data
    sources, their formats, and how End-Users can make the DPP or
    CE-RISE IS functionality more accessible with the support of
    existing or new requirements for Automation and Workflow within
    existing tools, tool chains.

2.  Screening and analytics for large Bills of Materials such as the
    ability to declare against restricted substances, recycled content,
    critical and strategic materials, and other SEE impacts

3.  Large-scale tasks for updating and monitoring DPP information across
    multiple DPPs including roll-ups of information and calculations.
    Some of these User Stories will be used to support the scope of the
    CE-RISE IS, but End-Users could envision a wider scope which could
    be supported by DPP system providers.

## Stakeholders to the System Architecture

Here, we introduce the primary system stakeholders. These stakeholders
are individuals, groups, or organizations with an interest in
architectural decisions and outcomes. Architecture stakeholders are not
the same as the intended users of the Information system. Intended users
of the Information System give input to requirements regarding expected
system features, but often do not impose requirements on the
architecture, i.e. the specific configuration of front-end, middle, and
back-end design delivering the intended features.

Table 4 provides an overview of the primary system stakeholders who may
fall into one or more of the following categories:

-   Individuals and organizations should be knowledgeable about the
    system architecture.

-   Parties who must be persuaded or confident in the architecture.

-   Individuals and organizations who will collaborate with the
    architecture or its implementation.

-   Those who require architectural documentation to support their work.

-   Decision-makers responsible for system-related decisions and
    development.

Table 4 reflects the status per June 2024. An up-to-date version is
maintained here: <https://git.nilu.no/ce-rise/architecture>

<span id="_Ref170489791" class="anchor"></span>Table 4: CE-RISE
information system architecture stakeholders

<table>
<thead>
<tr class="header">
<th><strong>Name</strong></th>
<th><strong>Expectations</strong></th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td>NILU’s development team</td>
<td><ul>
<li><p>Clear and measurable implementation guidelines that outline the structure and functionality of CE-RISE Information System</p></li>
</ul></td>
</tr>
<tr class="even">
<td>UiO’s development team</td>
<td><ul>
<li><p>Clear use case(s) for the search functionality</p></li>
<li><p>An architecture capable of interoperating with the search functionality service</p></li>
</ul></td>
</tr>
<tr class="odd">
<td>CE-RISE WP4 and system</td>
<td><ul>
<li><p>Clear definition of methods behind features as basis for technical implementation</p></li>
<li><p>Clear definition of required data sources, interfaces and requirements for building blocks</p></li>
</ul></td>
</tr>
<tr class="even">
<td>WP2 lead concept developers (EMPA and BRGM)</td>
<td><ul>
<li><p>Delivery of anticipated RE module functionality by implementing features concept</p></li>
</ul></td>
</tr>
<tr class="odd">
<td>WP3 lead SEE module concept developers (Leiden University)</td>
<td><ul>
<li><p>Delivery of anticipated SEE module functionality through implementation in the Information System</p></li>
</ul></td>
</tr>
<tr class="even">
<td>WP5 lead case study</td>
<td><ul>
<li><p>A system made available through a front-end usable in business/industrial workflows</p></li>
<li><p>Delivery of anticipated functionality in line with WP2 and WP3 concept and anticipated user input stories</p></li>
</ul></td>
</tr>
<tr class="odd">
<td>CE-RISE project lead and other academic and industrial partners</td>
<td><ul>
<li><p>Integration of all WPs inputs</p></li>
<li><p>Successful piloting activities</p></li>
<li><p>Delivery of high-quality system supporting the unique value proposition of CE-RISE</p></li>
</ul></td>
</tr>
<tr class="even">
<td>NILU’s management</td>
<td><ul>
<li><p>Support effectively the activities of CE-RISE</p></li>
<li><p>Delivery of high-quality system supporting the unique value proposition</p></li>
<li><p>An IT infrastructure allowing post-project exploitation</p></li>
</ul></td>
</tr>
<tr class="odd">
<td>DPP providers</td>
<td><ul>
<li><p>Clear definition of interfaces and requirements for building blocks in linking the information system with Circularise’s app</p></li>
</ul></td>
</tr>
<tr class="even">
<td>Toolchain software providers</td>
<td><ul>
<li><p>Clear definition of interfaces and requirements for building blocks</p></li>
<li><p>Delivery of high-quality system supporting the unique value proposition</p></li>
</ul></td>
</tr>
<tr class="odd">
<td>Potential Business (data aggregators and resellers) and Consumer users</td>
<td><ul>
<li><p>Sufficient documentation and standardization allowing for interoperability</p></li>
<li><p>Documented value proposition</p></li>
</ul></td>
</tr>
<tr class="even">
<td>The European Commission/ Health and Digital Executive Agency (HaDEA)</td>
<td><ul>
<li><p>Support effectively the activities of CE-RISE</p></li>
<li><p>Delivery of an information system enabling further exploitation and positive impacts</p></li>
</ul></td>
</tr>
</tbody>
</table>

## Quality Goals

The following section lists main quality goals for the system
architecture. These quality goals are related to important elements for
the main stakeholders of the CE-RISE information system, described in
the previous section. The quality goals pertain to the architecture and
are not to be confused with quality goals for the CE-RISE project. For
example, the requirements set by the intended users of the Information
System are primarily focused on features and are not requirements
specific to the system architecture, i.e. the implementation and
configuration of the digital infrastructure delivering the desired
features.

In prioritizing these quality goals, we focus therefore on expectation
by major stakeholders actively involved with the system architecture. In
addition, we purposefully limit the scope here to the version of the
information system delivered within the timelines and scope of the
CE-RISE project.

Table 5 presents 8 main quality goals to measure the system architecture
and implementation. They are grouped in 3 levels of priority, reported
with \* symbols (\*\*\* = highest priority).

<span id="_Ref169533657" class="anchor"></span>Table 5: Quality goals
for the CE-RISE Information System implementation

<table>
<thead>
<tr class="header">
<th><strong>Quality goal (priority with *)</strong></th>
<th><strong>Description</strong></th>
<th><strong>Importance for stakeholders</strong></th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td><ol type="1">
<li><p>Security ***</p></li>
</ol></td>
<td><ul>
<li><p>Ensure robust authentication and authorization mechanisms.</p></li>
<li><p>Maintain data privacy and compliance with relevant regulations.</p></li>
<li><p>Implement intrusion detection and prevention to enhance system security.</p></li>
</ul></td>
<td><ul>
<li><p>The value proposition of the information system is dependent on its capability of storing private business data. Security is a prime feature to create trust in the system. Users require assurance that their data is protected against breaches and unauthorized access.</p></li>
</ul></td>
</tr>
<tr class="even">
<td><ol start="2" type="1">
<li><p>Performance *</p></li>
</ol></td>
<td><ul>
<li><p>Achieve low-latency access to the system and data sources.</p></li>
<li><p>Optimize data processing and computation for efficient performance.</p></li>
<li><p>Ensure that the system can handle a growing user base and data volume without degradation in response times.</p></li>
</ul></td>
<td><ul>
<li><p>Stakeholders need a system that performs efficiently to ensure productivity and user satisfaction. Performance impacts the overall user experience and determines the system's capability to support business operations effectively, especially as user numbers and data volume grow.</p></li>
</ul></td>
</tr>
<tr class="odd">
<td><ol start="3" type="1">
<li><p>Reliability and Availability **</p></li>
</ol></td>
<td><ul>
<li><p>Provide high system availability to prevent downtime.</p></li>
<li><p>Ensure data integrity and reliability in data integration and processing.</p></li>
<li><p>Implement redundancy and failover mechanisms for fault tolerance.</p></li>
</ul></td>
<td><ul>
<li><p>Reliability and availability are critical for users who depend on the system for continuous business operations. Downtime or data loss can lead to significant disruptions and loss of trust in the system. Stakeholders expect consistent performance and quick recovery from failures.</p></li>
</ul></td>
</tr>
<tr class="even">
<td><ol start="4" type="1">
<li><p>Usability **</p></li>
</ol></td>
<td><ul>
<li><p>Design user interfaces for authentication and access control that are user-friendly.</p></li>
<li><p>Document computed information in a clear and understandable manner for users and for the development of services re-using the information.</p></li>
</ul></td>
<td><ul>
<li><p>User-friendliness supports the value proposition of the information system. Users require interfaces that are easy to navigate and understand, which enhances user adoption and satisfaction. Clear documentation also aids in training and efficient use of the system.</p></li>
</ul></td>
</tr>
<tr class="odd">
<td><ol start="5" type="1">
<li><p>Scalability and transferability ***</p></li>
</ol></td>
<td><ul>
<li><p>Design the architecture to scale horizontally and vertically to accommodate increased user and data demands.</p></li>
<li><p>Ensure that the system can handle the integration of additional external data sources seamlessly.</p></li>
</ul></td>
<td><ul>
<li><p>The information system should be capable of accommodating a rapidly growing user base post-project. This may require transfer towards cloud solutions as a next step for exploitation. Stakeholders need assurance that the system can grow and adapt to future needs without significant rework.</p></li>
</ul></td>
</tr>
<tr class="even">
<td><ol start="6" type="1">
<li><p>Maintainability***</p></li>
</ol></td>
<td><ul>
<li><p>Design the system components with sufficient and clear documentation as well as automated testing routines where appropriate</p></li>
<li><p>Design the system to allow for easy modifications and corrections in response to changes in requirements</p></li>
</ul></td>
<td><ul>
<li><p>Rapid maintenance allows for a dynamic exploitation of the system. Stakeholders value maintainability for its impact on long-term system sustainability and cost efficiency. It ensures that the system can evolve and improve over time without excessive downtime or resource expenditure.</p></li>
</ul></td>
</tr>
<tr class="odd">
<td><ol start="7" type="1">
<li><p>Data Quality *</p></li>
</ol></td>
<td><ul>
<li><p>Enforce data quality standards in data integration, transformation, and enrichment.</p></li>
<li><p>Continuously monitor and improve data quality to ensure the accuracy and reliability of computed information.</p></li>
</ul></td>
<td><ul>
<li><p>High data quality is essential for stakeholders to make informed decisions based on reliable and accurate information. Ensuring data integrity and consistency enhances the credibility and utility of the system, which is crucial for stakeholders who rely on precise data for their operations.</p></li>
</ul></td>
</tr>
<tr class="even">
<td><ol start="8" type="1">
<li><p>Compliance **</p></li>
</ol></td>
<td><ul>
<li><p>Ensure that the system architecture complies with relevant industry standards and regulations, particularly in the areas of data privacy and security.</p></li>
<li><p>Keep scientific models and data pipelines up to date with the latest research and certifications.</p></li>
</ul></td>
<td><ul>
<li><p>Compliance with standards and regulations protects stakeholders from legal and financial repercussions. It ensures that the system adheres to best practices in data privacy and security, which is particularly important for stakeholders in regulated industries. Keeping models updated maintains system relevance.</p></li>
</ul></td>
</tr>
</tbody>
</table>

# ARCHITECTURE CONSTRAINTS

Constraints are requirements that limit the discretion of software
architects when making design, implementation, or development process
decisions. They may extend beyond individual systems and apply to entire
organizations or value chains. They could be technical, organizational,
and political.

They are key for architects to have a precise understanding of where
they have creative freedom in their design choices and where they are
obliged to adhere to these constraints. They are also key, from a more
dynamic perspective, in guiding the agile process of continuously
revising and improving the system architecture. Table 6 reports the 5
main constraints to the system architecture. They are discussed in
detail in the remaining sections of this Chapter.

<span id="_Ref169534764" class="anchor"></span>Table 6: Constraints to
the architecture

<table>
<thead>
<tr class="header">
<th><strong>Section</strong></th>
<th><strong>Constraint</strong></th>
<th><strong>Type</strong></th>
<th><strong>Solution</strong></th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td>3.1</td>
<td>Evolving DPP Legal Status and Standards</td>
<td>legal/political</td>
<td>The system architecture can remain resilient in the face of evolving DPP legal status and standards while promoting openness to multiple DPP providers in the field.<br />
This ensures long-term sustainability and adaptability in the ever-changing landscape of digital product passports.</td>
</tr>
<tr class="even">
<td>3.2</td>
<td>Compliance with Data Protection Legislation</td>
<td>legal</td>
<td>The system architecture ensures that it operates within the boundaries of data protection legislation, safeguarding user data and privacy rights throughout the entire system's lifecycle.</td>
</tr>
<tr class="odd">
<td>3.3</td>
<td>Budget Constraints for Autonomous Key Partners</td>
<td>organizational</td>
<td>The system architecture allows for the effective collaboration and integration of autonomous key partners while respecting their budget constraints and maintaining a clear focus on defined inputs/outputs and quality requirements.</td>
</tr>
<tr class="even">
<td>3.4</td>
<td>Intellectual Property Management</td>
<td>legal</td>
<td>The system architecture ensures that the management of intellectual property is strategically planned, minimizing potential risks to the system's unique value proposition and enabling the development of a sustainable long-term business model beyond the initial development phase.</td>
</tr>
<tr class="odd">
<td>3.5</td>
<td>Technology stack available during the project</td>
<td>technical</td>
<td>The system needs to be implemented on the technology stack available to NILU in the project period.</td>
</tr>
</tbody>
</table>

## Evolving DPP Legal Status and Standards

The legal status and standards governing Digital Product Passports
(DPPs) are in a constant state of evolution and development. Regulatory
requirements and industry standards related to DPPs may change over
time. This constraint necessitates that the system architecture remains
adaptable and flexible to accommodate these evolving legal and
interoperability standards.

Implications:

1.  The system must be designed with modularity and flexibility in mind
    to allow for seamless integration of updated DPP standards and
    compliance requirements.

2.  Regular monitoring and awareness of changes in DPP-related
    regulations and standards are essential to ensure ongoing system
    compatibility.

3.  Documentation and version control mechanisms should be in place to
    track changes in DPP-related legal requirements and standards.

4.  The system should incorporate a mechanism for timely updates and
    adjustments to adhere to the latest DPP standards.

### DPP Provider Agnosticism

The system must also be agnostic when it comes to DPP providers. It
should remain open and adaptable to accommodate any DPP provider that
meets the evolving legal and interoperability standards, fostering a
competitive and dynamic landscape in the DPP field.

Implications:

1.  The architecture should be designed to allow for easy integration of
    DPP services from various providers.

2.  Clear guidelines and interfaces should be established to facilitate
    the onboarding of new DPP providers.

3.  The system should maintain compatibility with multiple DPP provider
    APIs and data formats.

By addressing these constraints, the system architecture can remain
resilient in the face of evolving DPP legal status and standards while
promoting openness to multiple DPP providers in the field. This ensures
long-term sustainability and adaptability in the ever-changing landscape
of digital product passports.

## Compliance with Data Protection Legislation

The system must adhere to data protection legislation and regulations
that govern the handling, storage, and processing of user data. This
constraint emphasizes the importance of safeguarding user privacy and
ensuring that the system operates in full compliance with applicable
data protection laws.

Implications:

1.  Data protection laws, such as GDPR (General Data Protection
    Regulation) in the European Union, must be thoroughly understood and
    integrated into the system's design and operation.

2.  Privacy by design principles should be applied, ensuring that data
    protection measures are considered at every stage of system
    development.

3.  User consent mechanisms for data collection and processing must be
    implemented, allowing users to exercise control over their data.

4.  Robust data encryption and access control mechanisms should be in
    place to protect sensitive user information.

5.  Regular audits and assessments should be conducted to ensure ongoing
    compliance with data protection legislation.

6.  Adequate documentation and records of data processing activities
    should be maintained for compliance reporting and auditing purposes.

By adhering to this constraint, the system architecture ensures that it
operates within the boundaries of data protection legislation,
safeguarding user data and privacy rights throughout the entire system's
lifecycle.

## Budget Constraints for Autonomous Key Partners

The system architecture must account for budget constraints and
financial autonomy among key partners responsible for developing
building blocks – understood here to be NILU, University of Oslo for the
information system and Circularise for the DPP solutions offered through
the CE-RISE project.

Implications:

1.  The system architecture should establish clear and well-defined
    interfaces for communication and data exchange with the autonomous
    key partners.

2.  Quality requirements and expectations for the outputs generated by
    these partners should be explicitly documented to ensure alignment
    with the system's overall goals.

3.  Budget allocation and management within the black-boxed key partner
    components should be the responsibility of the respective partners,
    ensuring their autonomy.

4.  Regular communication and collaboration mechanisms should be in
    place to facilitate coordination between the lead system developer
    (NILU) and the autonomous partners regarding budget considerations.

5.  The system architecture should be designed to be flexible and
    adaptable to accommodate changes or updates from these partners
    without requiring detailed knowledge of their internal budget
    allocation.

By addressing this constraint, the system architecture allows for the
effective collaboration and integration of autonomous key partners while
respecting their budget constraints and maintaining a clear focus on
defined inputs/outputs and quality requirements.

## Intellectual Property Management

The system architecture must consider the management of intellectual
property (IP) related to its components, as this can have significant
implications on the system's unique value proposition and its long-term
viability as a sustainable business model beyond the development phase.

Implications:

1.  Clear agreements and policies should be established regarding
    ownership, licensing, and usage rights of IP associated with system
    components.

2.  Consideration should be given to protecting valuable IP that
    contributes to the system's unique features and value proposition.

3.  The architecture should accommodate mechanisms for IP sharing,
    licensing, or collaboration with external entities when necessary.

4.  Business models and revenue-sharing arrangements related to IP
    should be defined and aligned with the long-term sustainability of
    the system.

5.  Provisions for IP protection, such as patents, copyrights, or trade
    secrets, should be considered as applicable to safeguard the
    system's innovations and competitiveness.

By addressing this constraint, the system architecture ensures that the
management of intellectual property is strategically planned, minimizing
potential risks to the system's unique value proposition and enabling
the development of a sustainable long-term business model beyond the
initial development phase.

## Availability of Technology Stack

The technologies and tools used in the software system must be
accessible and usable by the organizations responsible for developing,
deploying, and maintaining the system. Within the CE-RISE project period
this is NILU and the University of Oslo. This constraint ensures that
the chosen technologies align with the capabilities, resources, and
existing infrastructure of these organizations.

Implications:

1.  Resource optimization ensures the use of current hardware, software,
    and infrastructure, avoiding redundant investment. In addition, it
    utilized the existing knowledge and skills of the NILU development
    team.

2.  The existing technology stack enables faster deployment. It speeds
    up project timelines as the development team is already familiar
    with the technology stack.

3.  Utilizing the existing technology stack builds on established
    support structures available at the institute. Benefits from
    pre-existing support and maintenance procedures ensure resolution of
    issues.

4.  Less flexibility in testing new solutions. Architectural decisions
    relating to, e.g., implementation languages, deployment
    possibilities are constrained by the current technology and know-how
    available at the institute.

By addressing this constraint, the system architecture allows for the
most effective development and testing, allowing for agile
implementation.

# SYSTEM SCOPE AND CONTEXT

In the following sections we describe the business context (section 4.1)
and the technical context (section 4.2). The business context shows the
envisioned and potential links between the Information System and its
“environment”. We highlight likely communication partners (either users
or software systems). This is reflected by subsections on software
systems, dataset providers for inclusion in the Information System and
an extensive description about intended users and the software systems
they operate. The technical interfaces between the system and its
environment are outlined in the technical context section.

## Business Context

Departing from the CE-RISE concept shown in Figure 1, Figure 2 shows the
context of the CE-RISE information system, indicated in black. It
illustrates various Information System users and stages in the value
chain, ranging from extraction to producers or Original Equipment
Manufacturers (OEMs), and finally RE-actors. Repair, recycling and
refurbishing are highlighted as examples of RE-actors in Figure 2. In
each stage of the value chain, the value chain actors utilize company
data and software systems to manage information in relation to the raw
material, component or product. Some of this information is included in
the DPP, for which the DPP service provider provides the infrastructure
linking DPPs together.

Figure 2 highlights the links between the DPP service provider and the
CE-RISE Information System and example uses are from different actors in
the value chain, such as the checking of product documentation on
disassembly, are included. External data sources, as well as potential
links to other software systems are highlighted.

<img src="media/fig2.png" style="width:6.5in;height:5.13889in" />

<span id="_Ref166589794" class="anchor"></span>Figure 2: CE-RISE
information system concept and interplay between different users.

### Software Systems Interacting with the Infrastructure

#### Digital Product Passport (DPP) services

Output from software systems / input to CE-RISE: DPPs and linked DPPs

DPP service providers, represented in the project by Circularise, will
make available Digital Product Passports data for products supported by
their infrastructure. Users may opt to write CE-RISE information system
generated results to their DPP.

#### Other software systems

In this document, we open up for the possibility of collecting data
through API from other software systems. These systems may be used by
Information System users, but also can consist of general external
resources available through API, e.g. a link to a frequently updated
database. The extent to which output from other software is required
will depend on the final methods and workflows implemented and the
corresponding data availability and requirements for each workflow.

### CE-RISE initial dataset and model providers

The CE-RISE digital infrastructure is built upon a set of external data
sources, and some of them are developed by the project. Within the scope
of the project, these data sources are incorporated into the databases
of the CE-RISE information system. The following sections give an

#### Life Cycle Inventory (LCI) provision

***Input to CE-RISE:*** Life Cycle Inventory data.

Within the project, Life Cycle Inventory data is assembled from a
variety of sources outlined in Deliverable 3.1. These data are
considered essential for supporting the system's operation, in
particular in enabling the SEE module to compute environmental impact
indicators.

#### Environmentally Extended Input Output (EEIO) Tables 

***Input to CE-RISE:*** Environmentally Extended Input Output or Supply
Use Tables

Environmentally Extended Input Output (EE-IO) tables and Supply Use
(SUT) tables are created and sourced from a variety of sources outlined
in Deliverable 3.1 as part of the activities under Task 3.2 in WP3.
Together, these economic environmental accounting tables are expected to
contribute valuable information for environmental impact assessments
within the system, by providing up-to-date information on national
backgrounds.

#### BERT LLM 

***Input to CE-RISE:*** BERT LLM

The BERT (Bidirectional Encoder Representations from Transformers) large
language model is expected to be finetuned to enable the semantic search
functionality of the CE-RISE information system. BERT is described here
(Devlin et al., 2019) and available here (Google Research, 2024).

### User personas

Software designers and developers use personas as guidance when
developing a software product that fits the needs of a specific type of
end-user or multiple types of end-users to decide whether to add
specific features, interactions or functions, or visuals for enhanced
end-user experience. Reflecting on typical personas aids the development
team in the following ways (Salimi, 2024):

-   It increases the developer’s awareness of the type(s) of end-users
    of their product.

-   It improves the value and integrity of the product.

-   It produces a more focused and streamlined product.

-   It enables developers to determine more easily what to include.

In this work, we have consolidated a list of *primary* and *secondary*
personas based on the capture of User Stories from software end-user
project partners: Primary Personas are expected to interact directly
with the CE-RISE IS. Secondary personas may prepare information or data
for the main end-users or are a beneficiary of output or outcomes of the
Information System. Table 7 provides an overview of the considered
personas mapped against the number of user stories written from the
persona perspective.

<span id="_Ref170206058" class="anchor"></span>Table 7: Primary and
secondary user personas of the Information System.

| **Primary persona**               | **Reflected in \# of user stories** | **Secondary personas**       | **Reflected in \# user stories** |
|-----------------------------------|-------------------------------------|------------------------------|----------------------------------|
| Environmental & Regulatory Expert | 54                                  | Product Design & Development |                                  |
| Production                        | 32                                  | Modelling & Simulation       |                                  |
| Physical Separator – Technician   | 12                                  | Materials Engineer/Scientist |                                  |
| Repair – Technician               | 8                                   | Customer – General Public    | 1                                |
| Physical Collector – Technician   | 2                                   | Sales & Marketing            | 6                                |
| Compliance Consultant             | 12                                  | Consumer – General Public    |                                  |
| Manufacturing Engineer/Scientist  | 6                                   | Policy Experts               |                                  |
| Procurement                       | 12                                  | IT – Administrator           | 1                                |
| Researchers & Educators           |                                     | Software – Product Manager   |                                  |

The primary persona profiles are summarized in Table 8 and reflect their
typical job titles, career experiences, job skills, high-level software
and data requirements as end-users of the CE-RISE IS, and architectural
component(s) of interest. A full assessment of end-user *maturity* in
terms of data awareness and availability (e.g. environmental data),
digitalization and related software tools, supply chain management
processes, or people development in areas of environmental assessment,
is beyond the immediate scope of this first research phase of the
project.

Multiple personas could be represented within each value chain member of
the CE-RISE project, and we have spoken with specific project members
representing those personas as indicated in Table 8. Some of this
information was captured by WP1 surveys.

<span id="_Ref170202356" class="anchor"></span>Table 8: CE-RISE primary
personas

<table>
<thead>
<tr class="header">
<th><strong>Persona</strong></th>
<th><strong>Job Titles</strong></th>
<th><strong>Job Role</strong></th>
<th><strong>Software Tool Licenses</strong></th>
<th><strong>Software &amp; Data Requirements</strong></th>
<th><strong>CE-RISE Component</strong></th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td>Environmental &amp; Regulatory Expert</td>
<td>Sustainability / Circularity/ Environmental Affairs / Climate / CSR … Manager / Product / Analyst / Specialist</td>
<td>Reporting or declarations against all regulations (CSRD, EPR, ESRS, …) (sustainability, carbon footprinting, social responsibility, circular economy, renewable energy, CRM, SRM, …); strategic planning; stakeholder management; policy engagement; internal / external communication</td>
<td>LCA and related reporting systems for primary data including product lifecycle management (PLM) and enterprise resource management (ERP, incl. SAP) including supply chain data; machine generated data (e.g. energy usage); Excel; BoM screening; industry/regulatory standard secondary datasets</td>
<td><ul>
<li><p>User Interfaces</p></li>
<li><p>Curated datasets</p></li>
<li><p>Pre-scripted protocols</p></li>
<li><p>Maintained software and data with legal accountability</p></li>
<li><p>Automation of data handling and workflows</p></li>
<li><p>Interoperability</p></li>
</ul></td>
<td><ul>
<li><p>RE-Module</p></li>
<li><p>SEE-Module</p></li>
<li><p>Conformity</p></li>
</ul></td>
</tr>
<tr class="even">
<td>Production</td>
<td>Production / Manufacturing / … Manager / Technician</td>
<td>Quality assurance, diagnostics, disassembly, repair, refurbish, remanufacture, equipment and asset management (incl. robotics)</td>
<td>Digital and electronics diagnostics (e.g. Aiken), physical testing, robotics and machine control</td>
<td><ul>
<li><p>As Above</p></li>
</ul></td>
<td><ul>
<li><p>RE-Module</p></li>
</ul></td>
</tr>
<tr class="odd">
<td>Compliance Consultant</td>
<td>Producer / Product / Environmental/ Sustainability / Take Back … Compliance / Consenting / Management</td>
<td>Reporting Quality Assurance, audits (sustainability, waste mngt., environmental protection, material flow analysis, reporting mngt., circular economy)</td>
<td>As above for Environmental &amp; Regulatory experts</td>
<td><ul>
<li><p>As Above</p></li>
</ul></td>
<td><ul>
<li><p>RE-Module</p></li>
<li><p>SEE-Module</p></li>
<li><p>Conformity</p></li>
</ul></td>
</tr>
<tr class="even">
<td><p>Procurement</p>
<p>)</p></td>
<td>Commodity / Sourcing / Purchasing … Manager, Specialist</td>
<td>Bought-in product requirements and supplier identification, purchasing, E-commerce, cross-functional leadership, supply chain mngt.</td>
<td>Procure-to-Pay (P2P) software, Purchasing, Procurement, Request for Proposal (RFP), Accounts Payable (AP) &amp; Spend Analysis</td>
<td><ul>
<li><p>As Above</p></li>
</ul></td>
<td><ul>
<li><p>RE-Module</p></li>
<li><p>SEE-Module</p></li>
<li><p>Conformity</p></li>
</ul></td>
</tr>
<tr class="odd">
<td>Manufacturing Engineer/Scientist</td>
<td>Materials and Manufacturing Process</td>
<td>Capability acquisition, Design for X, product development, modelling, simulation, Six Sigma, Lean Manufacturing, Root Cause, …</td>
<td>CAD, PLM, finite element, machine control (incl. robotics), characterization and physical testing, machine learning / AI</td>
<td><ul>
<li><p>As Above</p></li>
<li><p>Access for scripting, coding</p></li>
<li><p>Data curation tools for dissemination</p></li>
</ul></td>
<td><ul>
<li><p>RE-Module</p></li>
</ul></td>
</tr>
<tr class="even">
<td>Researcher</td>
<td>Project / Program / Policy … Expert / Lead</td>
<td>Environmental science, ecology, climate change, societal change mass/ substance flow analysis, data analysis, modelling, policy</td>
<td>As above for Environmental &amp; Regulatory experts</td>
<td><ul>
<li><p>As Above</p></li>
<li><p>Access for scripting, coding</p></li>
<li><p>Data curation tools for dissemination</p></li>
</ul></td>
<td><ul>
<li><blockquote>
<p>Data access and analysis tools</p>
</blockquote></li>
</ul></td>
</tr>
</tbody>
</table>

### Other Software Systems Used by CE-RISE IS Users

Although a *general* persona for each job/role is presented in the
previous section, the role might change significantly across sectors,
business entity position within the value chain (upstream vs. downstream
actors), and the size of the business in terms of human resources and/or
revenue. This is of particular relevance when considering the software
tools the persona have access to. Below, examples wherein a
‘one-size-fits-all’ persona does not hold true:

-   Roles in enterprises with high revenue tend to have more and/or
    enterprise-level software licenses access to manage traceability of
    a variety of data/information sources, and the roles within the
    business are often more siloed (a persona performs specific tasks in
    a licensed software) leading to improved data accuracy and process
    efficiencies. A persona might have access to very different software
    tools, data, skills and processes (summarized in Table 9).

-   Materials Supply vs Product Manufacturing businesses will have
    differences in software for digitalization strategies (e.g. PLM is
    used by product manufacturing, but not necessarily by materials
    suppliers, as expressed in Table 9).

-   Businesses with high requirements on traceability for qualification
    and certification (e.g. automotive, aerospace, biomedical) tend to
    have high maturity for digitalization (reflected in enterprise
    software tool licenses and data curation, migration of legacy data
    systems), whereas businesses with less emphasis on these targets
    might not invest as significantly.

-   Sectors which emerged and/or grew significantly within the last
    decade (e.g. PV manufacturing) have been scaling-up in the same
    timeframe that digitalization and environmental reporting became
    accessible and mainstream requirements by clients and regulations,
    and hence these businesses began with these requirements planned as
    part of their scale-up efforts and tend to have a higher maturity.

-   Similarly, sectors which emerged because of the drive for circular
    economy business models will be willing to share more details about
    their environmental performance.

-   Enterprise-level tool suites will typically support higher degrees
    of interoperability and/or integration within their tool suite, and
    with 3<sup>rd</sup> partner tools for higher automation of tasks and
    workflows by end-users.

<span id="_Ref170203114" class="anchor"></span>Table 9: Access to
typical software solutions dependent on enterprise size

<table>
<thead>
<tr class="header">
<th><p><strong>Solution Data</strong></p>
<p><strong>Requirements Type</strong></p></th>
<th><p><strong>Software Data Management Solutions by</strong></p>
<p><strong>Data Requirements Type and Enterprise Size</strong></p></th>
<th></th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td></td>
<td><strong>Small-Medium</strong></td>
<td><strong>Large</strong></td>
</tr>
<tr class="even">
<td>Basic Product Data</td>
<td>Excel, Access, …</td>
<td>PLM</td>
</tr>
<tr class="odd">
<td>Material Data</td>
<td>MI, Excel</td>
<td>MI, PLM</td>
</tr>
<tr class="even">
<td>Ownership Data</td>
<td>CRM</td>
<td>CRM, PLM</td>
</tr>
<tr class="odd">
<td>Repair Data</td>
<td>CAD</td>
<td>CAD (CMS), PLM</td>
</tr>
<tr class="even">
<td>Sustainability Data – Carbon, Manufacturing, Distribution</td>
<td>LCA, MI, ERP, machine use data capture</td>
<td>LCA, MI, PLM, machine use data capture</td>
</tr>
<tr class="odd">
<td>Sustainability Data – Use Phase</td>
<td>DT, BMS, MI (EoL integrity)</td>
<td>DT, BMS, MI, PLM, SPDM</td>
</tr>
<tr class="even">
<td>Batch Data, including origin and recycling mix</td>
<td>MI, ERP</td>
<td>MI, PLM</td>
</tr>
</tbody>
</table>

Understanding the software systems used by potential Information System
users is of importance to outline potential linkages, e.g. as data
source, and to ensure that the information system is not unnecessarily
providing functionality already available elsewhere. Thus, it supports
the value proposition of the Information System.

Based on the User Stories, we can also start to understand the value
chain members and respective Personas interested in the proposed
components of the CE-RISE information system. Table 10 examines the
available information for the RE-Module and related calculations.
Expressed interest in the Repairability indicator/index, RE-Strategy
calculation, and validation of both against actuals extracted from
downstream actor contributions to the DPP was notably of interest for
the OEM and downstream actors. All value chain actors were interested in
SEE-impact calculations and compliance related to RE-strategies. Also
noteworthy is that the downstream actors for Repair/Refurbishment
(including the Repair Technician noted by the OEM) see the CE-RISE
RE-Module as an opportunity for technicians to gain specific information
and guidance related to their RE-Strategy, or to confirm the RE-Strategy
once chosen.

<span id="_Ref170203245" class="anchor"></span>Table 10: Interest of
Personas in CE-RISE IS capabilities

| **Feature**                                    | **Value chain and personas**                 |                                                                      |                     |                                                                                        |
|------------------------------------------------|----------------------------------------------|----------------------------------------------------------------------|---------------------|----------------------------------------------------------------------------------------|
|                                                | **Material**                                 | **Tier & OEMs**                                                      | **Compliance, PRO** | **Repair/ Refurbishment**                                                              |
| Repairability Index calculation (re-indicator) |                                              | Environment & Regulatory Expert, Sales & Marketing                   | Compliance          | Physical Separator Technician, Manufacturing Engineer                                  |
| RE-Strategy calculation                        |                                              | Environment & Regulatory Expert, M&S, Repair Technician, Procurement | Compliance          | Physical Separator Technician, Manufacturing Engineer, Repair Technician, Procurement  |
| Validate against actuals                       |                                              | As above                                                             | As above            | As above                                                                               |
| SEE-Impact                                     | Environment & Regulatory Expert, Procurement | Environment & Regulatory Expert                                      | t.b.d.              | Physical Separator Technician, Manufacturing Engineer, Environment & Regulatory Expert |
| SEE-Impact Compliance                          | Environment & Regulatory Expert, Procurement | Environment & Regulatory Expert                                      | Compliance          | Physical Separator Technician, Environment & Regulatory Expert                         |

### Business Context Summary

The Information System aims to add value to its users by adding
information and systemic insight into information contained in digital
product passports. Through the collection of user stories, nine primary
personas were identified, each representing different organizational
roles and with distinct responsibilities based on where in the value
chain the personas operate. In addition, we mapped the software
ecosystem used by the primary personas.

Three main avenues of data ingestion for the CE-RISE information system
are identified during initial development and operation: *i)* Data may
be made available to the system through the digital product passports,
*ii)* data may be provided by external software through API, and *iii)*
data (externally provided datasets) may be directly ingested in the
system database In addition, we would like to add here that the
Information System also expects to ingest data from system users in the
following ways: *iv)* data may be uploaded in the form of a file by a
system user, and *v)* data may be input through the system front-end
application by a system user.

## Technical Context

In the following section, we describe the technical interfaces linking
the CIS to its environment. We list envisioned interfaces with other
systems, highlight data formats, and technology/specific details to aid
system developers, architects and IT operations in the implementation of
the CIS.

All communication channels between the system and its context are built
upon, and depend on, REST APIs as their underlying communication
framework. API specification is an essential component of our project
development process. At the architectural level, we primarily define the
role and provide the strategic approach for guiding their development.
The detailed specifications and implementation details of these APIs are
handled within the development phase.

### DPP via REST API

The DPP I/O service provides a channel to retrieve Digital Product
Passport (DPP) data from external providers. It is used to request DPP
information. This API specifies the formats, data structures, and
communication protocols for transmitting DPP essential data into the
system. It serves as the foundational entry point for external DPP
providers to interact with the system, ensuring the seamless flow of
information critical to the Information System.

The DPP I/O API will also let us gather important information about
products in detail as they move through the supply chain, their history,
and what that are made of, based on the concept of linked DPPs. This
will be achieved using the following key principles:

1.  Comprehensive DPP Information: The DPP I/O collects all the
    necessary details about the products, such as what they are made of,
    how they use environmental resources, and other key facts, through
    the CE-RISE DPP scheme made available at Circularise. This will
    ensure that the information is complete and accurately reflects the
    products and their processes.

2.  Iterative Data Retrieval: The DPP I/O allows for the ongoing
    collection of product data by information system users. This means
    that as products go through various stages in their life cycle and
    move along the supply chain, their digital passports can be updated
    and improved. This process helps capture a full and accurate picture
    of the products.

### User Authentication

The CE-RISE IS front end is envisioned as a web application that allows
users to manage their profiles, view personalized content, and access
various services. We will take advantage of solutions such as Keycloak
for handling the complexity of authentication and providing robust
security features through OAuth 2.0. Enabling users to log in securely
and access protected resources is required in order to allow different
users of the Information System to track e.g. the number of products
passed through the Information System and to be able to identify them in
communications with the DPP service provider.

### User Data Upload

As part of the functionality of CIS and the scoping of the user stories,
we identified the necessity for user supplied data uploads. One way of
handling this is through an html form with a file-input element. This
element will allow users to upload the file (e.g. pdf, json, csv, xlsx,
docx) from their local file system and upload to the app. In addition to
a front-end component capable of selecting a document from the users
file system, a server-side endpoint URL is required through API. We will
consider the security implications associated with having a system that
allows file-upload (e.g. user should be prevented from uploading
documents containing malicious code). This may imply that not all file
formats are allowed.

### Other Data Ingestion through REST API

In addition, to data ingestion through file upload, or via the DPP, the
CE-RISE information system is expected to ingest data from other
sources. These sources are dependent as workflows and methods associated
with each of the system features are not yet defined as this is part of
a collaborative effort between Work Packages 2 and 4. However, we define
here the channel for such data ingestions through REST API.

### Internal API Gateway

The RE, SEE and product information services in the middleware layer are
made available through REST API. To simplify the web application
(client-side) code an internal API gateway is proposed wrapping around
the APIs of the microservices. The benefit of such an API gateway is
that it provides a single entry-point for all web application requests
and abstracts the internal microservices architecture from the client.
Routing will enable requests to be sent to the appropriate microservice,
i.e. the RE, SEE or any other service. An API gateway will also allow to
aggregate responses from multiple microservices into a single response.

While initially intended for internal use within the scope of the
CE-RISE project, such an API gateway eventually presents an exploitation
opportunity by allowing for increased interoperability post-project.
This API defines the formats, data structures, and communication
protocols necessary for external consumers and downstream systems to
access the valuable insights and results generated by the system. The
API gateway serves as the central interface responsible for delivering
essential system outputs, such as SEE Impacts and RE-indicator
calculations data. By utilizing the API gateway, users, partners, and
applications can seamlessly retrieve and integrate the Information
System outputs into their workflows or decision-making processes. It
acts as a key conduit for disseminating the system's value-added
information, enabling stakeholders to leverage the system’s unique
selling points and make informed decisions based on SEE Impacts and
RE-Criteria data.

This API is designed to not only define the formats, data structures,
and communication protocols for efficient data retrieval but also to
uphold key principles that enhance its utility and value:

1.  Efficient Data Access: The API gateway streamlines the process of
    accessing vital system outputs. By providing well-defined data
    formats and structures, it ensures that external consumers and
    downstream systems can efficiently retrieve SEE Impacts and
    RE-Criteria data.

2.  Seamless Integration: This API is architected to facilitate seamless
    integration with various workflows and decision-making processes.
    Users, partners, and applications can effortlessly incorporate the
    system's outputs into their operations, enabling data-driven choices
    and enhancing overall efficiency.

3.  Value Dissemination: Beyond being a conduit for data delivery, the
    API gateway is a channel for disseminating the system's value-added
    information. It empowers stakeholders to harness the unique selling
    points of the system by providing them with valuable insights
    derived from SEE Impacts and RE-Criteria data.

4.  Informed Decision-Making: Through the API gateway, stakeholders can
    make informed decisions based on the rich insights and results
    generated by the system. This empowers organizations and individuals
    to optimize their processes, reduce environmental impacts, and
    enhance their sustainability efforts.

# SOLUTION STRATEGY

## Solution Strategy Concepts

The system architecture adopts a modular approach, envisioning
'virtually' independent yet interconnected modules and building blocks.
It defines the complete data pipeline, establishes quality goals, and
sets maximum execution times. While it outlines interfaces between
components, the building blocks themselves are primarily viewed as black
boxes from the architectural standpoint. The architecture mandates
containerization and well-defined microservices, with the internal APIs
reserved for specification during the development phase. Architectural
focus lies in specifying what is needed (data, formats, and response
times), leaving the finer details of internal APIs to be addressed in
the development phase.

Our system architecture embraces modularity as a core principle. We
envision modules and building blocks that are interconnected yet
designed to function independently. The architecture takes a
comprehensive view, defining the entire data pipeline and establishing
stringent quality goals and maximum execution times. It also outlines
clear interfaces that facilitate communication between system
components.

However, we deliberately treat the internal building blocks beyond
C4-component level as 'black boxes' from the architectural perspective.
This means that while we specify their inputs, outputs, and the data
they require, we do not delve into the internal workings at the stage of
designing the system.

To ensure scalability and flexibility, we mandate containerization and
the use of well-defined microservices. These microservices will expose
specific APIs, which will be defined in detail during the development
phase and are outlined with example endpoints in section 6.1. The
architectural focus centers on specifying the essential elements
required for system operation- The specifics of the internal APIs,
including their functionalities and interactions, will be determined
during the development process, allowing for adaptability and efficient
implementation.

This design approach transforms the overall system into a flexible and
modular system empowered to dynamically adapt to varying user request
volumes and meeting acceptable execution speed standards. Additionally,
it allows for the dynamic selection of hardware configurations to
optimize performance in real-time, ensuring efficient resource
utilization and responsive user experiences.

This architectural approach not only enhances technical performance but
also significantly supports business development and economic
accounting. It enables the system to operate within budget constraints
during both development and runtime phases. Moreover, it empowers the
computation of variable costs associated with the added-value
information provided by the system. This capability ensures efficient
cost management while delivering valuable insights, making the system a
cost-effective and strategically sound solution for system managers and
end users.

Departing from the system scope in presented in Figure 1 and Figure 2,
we highlight the modular approach in Figure 3. In the Figure, we present
the Information System as a set of interconnected microservices. On the
left-hand side, is reflected the RE, SEE and Product information module
concepts as introduced in Figure 1. Each module is broken down here to
incorporate front-end elements as well as the databases required to
enable the functionality and features of the modules. The right-hand
side of Figure 3 contains support infrastructure, such as the search
service, data ingestion routines and user authorization and
authentication software. Finally, we exemplify important external links,
such as the link with Circularise, potential API links to other software
systems for data ingestion, or the upload of files exported from other
software.

<img src="media/fig3.png" style="width:6.23599in;height:3.68194in" />

<span id="_Ref170115575" class="anchor"></span>Figure 3: The CE-RISE
information system as a set of interconnected microservices, a front-end
application interface, and supporting databases.

## Technology Stack

The following Technology Stack is planned to be used to build and deploy
the CE-RISE Information System:

**Programming Languages and Frameworks:**

A variety of programming languages will be considered for the
development of the Information System, such as Java and Python,
primarily chosen as the languages of choice due to the previous
experience of the developing team with these languages. Vue.js was
chosen for our front-end development for similar reasons, in addition to
Vue.js technical advantages, particularly in handling dynamic content.

**Containerization:**

-   Docker: Used to containerize the web services, ensuring consistency
    across different development and deployment environments.

**Databases**

-   Neo4j: A graph database management system, for storing and querying
    highly connected data.

-   PostgreSQL: relational database system. It is ACID-compliant and
    supports foreign keys, joins, views, triggers, and stored
    procedures.

-   Qdrant vector database: A high-performance, open-source database
    designed for storing and managing vector embeddings, which allows
    for efficient similarity search and nearest neighbor search in
    applications such as machine learning, recommendation systems, and
    natural language processing.

**Version Control**

-   GitLab: Provides Git-based version control, allowing multiple
    developers to work on the same project without conflicts.

**Continuous Integration/Continuous Deployment (CI/CD)** (see Figure 4
below)

-   We will take advantage of GitLab functionalities to ensure a
    flexible CI/CD pipeline, suitable for CE-RISE Information complex
    development workflows that require detailed customization and a
    broad range of plugins and integrations.

-   Liquibase: Integrated into the CI/CD pipeline to manage database
    schema changes. It ensures that database changes are versioned,
    tracked, and applied consistently across all environments, from
    development to production.

**Additional Tools and Services**

-   Postman: API client that can be used for testing web services

-   Swagger: Tool for designing, building, and documenting and testing
    of the CE-RISE Information System’s RESTful APIs.

## Version Management and Release Strategy

We will adopt a release strategy based on semantic versioning
(SemVer [5]) at first, and subsequently transition to continuous
deployment once all services are available in a major release and
functional requirements are met. Semantic Versioning is a versioning
system based on a three-part numerical format, i.e. major.minor.patch.
It is adopted for software products where version compatibilities
between API integrations is critical. Major versions numbers change when
there are incompatible API changes, minor versions change when new
functionality is added in a backwards compatible manner, and the patches
occur when developers fix software bugs in a backwards compatible way.
In addition to the major.minor.patch distinction, semantic versioning
allows for distinguishing pre-release and build versions. Semantic
versioning is specified in an 11-point specification rules published on
the semantic versioning website <https://semver.org> (version 2.0.0).

Using semantic versioning allows the developers to track the full
dependencies of the different services, on each other, as well as on
external software systems. This is of importance in light of the
parallel development of the individual services described in Section
6.1. As the CE-RISE project and methods developed within the project
mature, functions can be added to the individual services in a backwards
compatible way. The CE-RISE information system is to maintain thorough
integration testing between the many parts of the system. Ideally it
triggers warnings if any change in one part turns out to be breaking for
any other part, or for the communication between them.

Once the first versions of all key features of the CE-RISE information
system are met, and the system is publicly available for testing and use
by the CE-RISE partners, we will employ continuous deployment. We
anticipate frequent feedback from CE-RISE participants on the CE-RISE
information system with regards to updated or improved functionality. To
allow for rapid iterations continuous deployment ensures that.
Continuous deployment does require the set-up of automated tests, that
test all functions for each service and integration tests. Only when all
tests are passed, changes (e.g. updates or added functionality) are
deployed to production. This approach is illustrated in Figure 4.

For managing DML (Data Manipulation Language) and DDL (Data Definition
Language) database schema updates as part of a continuous deployment
pipeline we will use tools such as Liquibase
<https://www.liquibase.com/>) to ensure that database changes are
consistently and safely applied across development and production
environments.

<img src="media/fig4.png" style="width:6.5in;height:3.69792in" alt="A screenshot of a computer Description automatically generated" />

<span id="_Ref170144309" class="anchor"></span>Figure 4: Deployment
dataflow based on GitLab and Liquibase

# BUILDING BLOCK VIEW

We define here the static decomposition of the system into building
blocks as well as their dependencies. We follow the C4 modelling method
for documenting architectures to break down the building blocks at
container and component level. Note that container level in C4
terminology does not highlight a Docker container, but rather any
deployable entity. C4 allows to present the building block of the
digital infrastructure in a hierarchical presentation, moving from a
high-level system overview to detailed descriptions of individual
containers. In Section 6.1, we present the Level 1 overview. This is a
“white box” (i.e. with components inside the box exemplified or defined)
description of the overall system together with “black box” descriptions
of all the containers. For the RE, SEE and Product Information service
we include the top-level features as described in Deliverable 2.1. The
final list of features of each of the containers will be more detailed
and extensive. An initial mapping, taking into account features
collected from User Stories and WP2 inputs is available here:
<https://git.nilu.no/ce-rise/architecture>

In Section 6.2, we zoom into the individual containers and allow for
identification of the individual links of the components into the
containers - a level 2 description. This section contains the white box
description of most of the containers selected in level 1, together with
black box descriptions of their internal building blocks.

## C4 – Container View

At the first level of system decomposition, we specify 15 C4-containers.
These containers are distinct in their roles and responsibilities and
deliver a distinct set of functionalities. However, some of the features
of the CE-RISE information system require containers to act in concert,
and links exist between the containers to ensure the seamless operation
of the entire system. Thus, the building blocks do not operate in
isolation but rather, they rely on one another to orchestrate data
management and processing and together, they define the system's
fundamental structure and data pipeline. Some key relations are drawn in
the Figure above. Further linkages are specified in section 6.1.9 as
well as by the sequence diagrams in Chapter 7, showing how the system
can support key generalized user stories collected from CE-RISE
participants.

<img src="media/fig5.png" style="width:6.24306in;height:7.87345in" />

<span id="_Toc170654524" class="anchor"></span>Figure 5: White box
C4-Container view of the CE-RISE information system.

### DPP I/O (black box)

The DPP Input/Output service is designed as the tool for accessing and
retrieving Digital Product Passport (DPP) data. It is the primary link
between the digital infrastructure and DPP service providers. While its
detailed definition is deferred to the development phase, its high-level
purpose is to facilitate the retrieval of digital product passports from
DPP service providers, transform these into the internal CE-RISE DPP
data model, and make available parts of this DPP to the rest of the
system. The container ensures data format compatibility and meets data
source-specific requirements. In the long run, we could possibly design
this to let any DPP service provider adopt to our “type” and do the work
themselves to adapt to using our service.

The following table lists the top-level features of the DPP I/O service.

<span id="_Toc170654505" class="anchor"></span>Table 11: Features of the
DPP I/O service

| **Identified features DPP I/O service** |                                                                                           |
|-----------------------------------------|-------------------------------------------------------------------------------------------|
| F1-DPP                                  | Provide connection to external DPP service providers (Circularise)                        |
| F2-DPP                                  | Transform incoming DPPs to internal DPP data scheme                                       |
| F3-DPP                                  | Make available (parts of) DPPs to other the other parts of the CE-RISE information system |

### RE Service (black box)

The RE-service is intended to provide the functionality to assess and
compute RE-indicators and associated criteria for products, evaluating
their suitability for various RE-strategies such as reuse, repair,
repurpose, refurbishment, remanufacture, or recycling. It leverages data
from Digital Product Passports (DPPs) and other sources, including user
supplied data, to determine the end-of-life potential of products, with
a focus on enhancing the circularity of components, materials, or
elements. By calculating RE-indicators based on established
methodologies and assessing the presence of critical and secondary raw
materials using information from DPPs and the European CRM list, the
RE-service provides essential indicators to support decision-making. Key
features are included in Table 12 below.

<span id="_Ref170592459" class="anchor"></span>Table 12. Features of the
RE service

| **Identified features- RE-service** |                                                                      |
|-------------------------------------|----------------------------------------------------------------------|
| F1-REM                              | Calculate RE-indicators for products considering CRM and SRM content |
| F1.1-REM                            | Calculating the CRM intensity within the product                     |
| F1.2-REM                            | Displaying EOL collection and treatment facilities                   |
| F1.3-REM                            | Calculating the disassembly effort of the product                    |
| F2-REM                              | Calculate the circularity rate of products                           |

### SEE Service (black box)

The SEE service is dedicated to the computation of socio-economic and
environmental impact indicators. These consist primarily of Life Cycle
Impact Assessment (LCIA) indicators covering the environmental
perspective. In addition to the LCIA indicators the module contains
calculation routines for recycled content, and socioeconomic footprints
estimations, such as a labor footprint. Primary input parameters for the
SEE module are the relevant LCI parameters from the DPP product and
supply chain, as well as links to a background LCI and background SUT
database. Key features are listed in Table 13 below.

| **Identified features- SEE service** |                                                                                                          |
|--------------------------------------|----------------------------------------------------------------------------------------------------------|
| F1-SEM                               | Calculate Life Cycle Assessment (LCA) of a product or components (environmental calculation)             |
| F2-SEM                               | Calculate impact associated with a specific RE-Strategy applied to a product (environmental calculation) |
| F3-SEM                               | Calculate the cost of RE-strategies                                                                      |
| F4-SEM                               | Measure social evaluation (relevant indicator e.g. job creation)                                         |
| F5-SEM                               | Measure overall sustainability assessment                                                                |

<span id="_Ref170592509" class="anchor"></span>Table 13: Features of the
SEE service

### Product Information (black box)

The Product Information service in the Information System includes
secondary features designed to support decision-making and ensure
quality and conformity. This module includes conformity assessment as a
main feature. It aims to ensure the quality of results provided by the
CE-RISE system, in particular in relation to the data included in the
Digital Product Passports. It may, for example, evaluate DPP information
against the CE-RISE data scheme criteria. Key features of the product
information service are shown in Table 14 below.

<span id="_Ref170592758" class="anchor"></span>Table 14: Features of the
product information service

| **Identified features- Conformity assessment** |                            |
|------------------------------------------------|----------------------------|
| F1-CAS                                         | Certify CE-RISE results    |
| F2-CAS                                         | Provide CE-RISE conformity |

### Semantic Search (black box)

As opposed to classical search, semantic search aims to understand
meaning behind search queries. By leveraging semantic technologies such
as Large Language Models (LLMs) and ontologies like the Web Ontology
Language (OWL/RDF), semantic search engines can interpret the context,
relationships, and semantics of the data. This enables advanced
capabilities such as concept-based search, entity recognition, and
semantic matching, allowing users to find information more efficiently.
Approaches using LLMs enhance natural language understanding, while
OWL-based ontologies provide structured knowledge representation,
facilitating precise and context-aware search functionalities. Key
features of semantic search are presented in Table 15.

<span id="_Ref170592729" class="anchor"></span>Table 15: Features of the
semantic search service.

| **F1-SRCH** | **Search through the different databases in the information system using natural language** |
|-------------|---------------------------------------------------------------------------------------------|
| F2-SRCH     | Understanding the meaning of search words and provide relevant results                      |
| F3-SRCH     | A search engine capable of general based search or semantic search                          |

### Databases (black box)

Several of the services in the Information System rely on databases. 6
main databases have been identified, each with their own purpose. For an
overview, see Table 16. LCI and SUT databases contain the
environmental-economic data required for gap filling the LCI inventory
model created in-site by the SEE service. The resulting model is used to
assess the overall environmental footprint of products.

The RE database is intended as a data repository for external generic
datasets required to enable the functionality of the RE-service. It
supports the RE Service in calculating metrics such as the repairability
index and circular economy rates, aiding in circularity analysis and
decision-making.

The Product Database stores computed information following from
Information System users, such as calculated RE-indicators or calculated
SEE-impacts associated with a product ID and associated with a specific
user account. This will allow the user to retrieve the accumulated
results for all the CE-RISE product systems.

The Vector Database supporting the search service is designed to store
vectorized representations of data elsewhere in the system, enabling
advanced search and retrieval capabilities. It supports the Search
Service by providing fast and efficient querying of large datasets,
leveraging the power of vector search to enhance data processing and
matching.

The User Database manages user-related information, including
authentication details, user profiles, and access permissions. It is
essential for handling user sessions, securing access to the
application, and ensuring that user interactions are properly
authenticated and authorized within the system.

<span id="_Ref170244204" class="anchor"></span>Table 16: Overview of the
identified databases in the CE-RISE information system.

| **DB**        | **Purpose**                                                                 |
|---------------|-----------------------------------------------------------------------------|
| LCI DB        | Used to store LCI datasets collected by the project                         |
| SUT DB        | Used to store SUT and MRIO datasets collected by the project                |
| RE DB         | Used to store relevant datasets for enabling the features of the RE service |
| Product DB    | Used to store output results associated with a product                      |
| Vector DB     | Used to enable semantic search on the system                                |
| User database | User for user management                                                    |

### Data Ingestion Service (black box)

The CE-RISE information system has a clearly defined data input through
the DPP I/O service. However, other datasets accessed by the API are not
included in this DPP I/O service to keep the DPP I/O separately managed.
For this reason, and as also described in section 4.2.4, the Information
System architecture contains a Data ingestion service, meant as a
placeholder for external datasets. At present, not many external data
services are identified in the conceptual Work Package 2. However, it is
expected that, as the methods for each of the features become available
in the course of the project, various data sources will be added.

### Web-App/Front-End (black box)

The web app is responsible for publishing system-generated data,
including SEE Impacts and RE-Criteria, to external consumers and
downstream systems. It acts as the main interface for user interaction.
We envision multiple views for the web-app, based on persona needs. For
example, accessing the web app via mobile phone may be necessary for
those personas requiring access to simple functionality of the
information system. However, other functionality, such as a detailed
inspection of information contained in the DPP is best done through a
personal computer. CE-RISE contains a dedicated task for front-end
development (task 4.5) where system developers can work together with
the case study partners to design the user interface and user
experiences. This will primarily be done through the creation of
wireframes and subsequent implementation by the developing team.

### API Gateway (black box)

As described in this Chapter and above, the RE, SEE and product
information services in the middleware layer are made available through
REST API. The internal API gateway is to provide a single entry-point
for all web application requests and separates the internal
microservices architecture from the client. Routing will enable requests
to be sent to the appropriate microservice, i.e. the RE, SEE or any of
the other services.

### Authentication and Authorization (black box)

Maintaining secure access across a multitude of microservices is
essential for both operational integrity and compliance with
regulations. This section describes our strategy for securing access
through sophisticated authentication and authorization mechanisms, with
Keycloak serving as the primary identity management system, possibly
supported by various identity providers.

***Authentication Strategy***

Keycloak will serve as the central hub for all authentication operations
within our architecture. It will be responsible for validating user
credentials and issuing security tokens for access management.
Keycloak's adaptability allows it to effectively interface with a range
of identity providers, such as Active Directory (AD), which is commonly
used in organizations with established Microsoft infrastructure. The
specific identity providers to be supported will be determined as the
project progresses. After successful authentication, Keycloak issues
JSON Web Tokens (JWT). These tokens securely encapsulate the user's
identity, roles, permissions, and the duration of token validity,
ensuring data protection through encryption.

***Authorization Strategy***

Following authentication, the system's authorization processes determine
the access levels for authenticated users. Each microservice
autonomously conducts its authorization checks, verifying the rights
within the bearer token against their specific policies before
processing any requests. This decentralized approach allows each
microservice to implement tailored security measures that align with
their unique operational needs.

Keycloak also plays a vital role in authorization; it coordinates the
synchronization of user roles and permissions among various identity
providers. This synchronization ensures that the permissions are a
current and accurate reflection of the user's roles, which are assessed
against the policies of each microservice to either allow or restrict
access.

***Security and Compliance***

All exchanges of sensitive data, including transactions involving
authentication tokens, are secured using Transport Layer Security (TLS)
to prevent data breaches and tampering. Additionally, our architecture
will be designed to meet relevant data protection regulations and
industry standards, necessitating ongoing monitoring and routine
security evaluations.

To proactively address potential security issues, our system will
incorporate logging and monitoring mechanisms. These tools track access
patterns and highlight unusual activities, aiding in the maintenance of
high security standards and enabling rapid response to any threats.

This comprehensive security framework will ensure that our microservices
architecture remains secure, compliant, and capable of adapting to
evolving organizational and technological landscapes, thereby supporting
our commitment to maintaining robust security and operational
efficiency.

## C4-Component Diagrams

In the following subsections we highlight the white box description of
selected individual containers outlined in Section 6.1. Each container
is described in a separate subsection, and the individual exemplified
components are shortly described. As an overarching principle, each of
the containers is made available through REST API, and example API
endpoints are presented in tables for the DPP I/O, RE, SEE, Product
Information, Semantic Search, and other data ingestion services. Within
CE-RISE, it is an ambition that most data to the system is supplied
through the DPP. However, in practice, we account for the possibility of
user-provided data, either through input forms in the front-end or
through file upload. For all white box descriptions in the following
section, this user-supplied data is assumed to be sent along with the
corresponding API requests.

### DPP I/O (white box)

This component diagram of the DPP I/O container provides a detailed
breakdown of the internal components of the DPP retrieval, highlighting
its interaction with the external DPP service provider and its role as
core DPP provider to the rest of the system. It ensures that data is
gathered and passed to subsequent components for further processing
within the system. Note that further DPP service providers could be
included to the system by extending the functionality of the link and
transformer components system, thus providing an opportunity for CE-RISE
to include multiple DPP service provider in the future

<img src="media/fig6.png" style="width:4.53472in;height:4.38685in" />

<span id="_Toc170654525" class="anchor"></span>Figure 6: C4-component
level diagram of the DPP I/O service.

#### DPP API

<span id="_Toc170654511" class="anchor"></span>Table 17: Example API
endpoints for the DPP I/O service

| **Endpoint**                        | **Returns**                                                        |
|-------------------------------------|--------------------------------------------------------------------|
| GET api/v1/dpp/{productId}          | Returns DPP from Circularise using Circularise credentials         |
| GET api/v1/tree/{productId}         | Tree of linked DPPs from Circularise using Circularise credentials |
| GET api/v1/materials/{productId}    | Returns material composition of product                            |
| GET api/v1/certificates/{productId} | Certificates stored in DPP                                         |
| GET api/v1/inventory/{productId }   | Inventory relevant data stored in the DPP                          |

#### Transformer

The DPP transformer serves as the central hub for collecting and
standardizing data from various DPP service providers, ensuring
uniformity and data quality. The Transformer component processes and
transforms the fetched DPP data into an internal standard format. It
makes the transformed parts of the DPP available to the rest of the
system.

#### Connector 

The Connector component is responsible for extracting the requested
information from the DPP service provider. It fetches the DPP data and
prepares it for further processing by the Transformer. The Connector
does this by sending requests to the external DPP Service Provider to
retrieve the full DPP. It then passes the fetched data to the
Transformer for internal standardization.

### RE-Service (white box)

Figure 7 illustrates the components of the RE Service, which is
responsible for calculating various metrics related to product
RE-indicators as well as circularity. It includes components such as the
API, RE-indicator, Disassembly Effort, and Circularity, each performing
specific tasks. The API Gateway routes external requests to the RE
Service, where they are processed by the respective components. The RE
Service interacts with the DPP I/O container to fetch necessary data
from the DPP Service Provider. This setup ensures efficient calculation
and retrieval of RE metrics.

<img src="media/fig7.png" style="width:4.26389in;height:3.67529in" />

<span id="_Ref170226838" class="anchor"></span>Figure 7: Component
diagram of the RE-service

#### RE-service API

<span id="_Toc170654512" class="anchor"></span>Table 18: Example API
endpoints for the RE-service

| **Endpoint**                               | **Returns**                                                           |
|--------------------------------------------|-----------------------------------------------------------------------|
| GET /api/v1/re-indicator/{productid}       | Fetches the RE indicator for a specific product.                      |
| GET /api/v1/disassembly-effort/{productid} | Calculates the disassembly effort calculation for a specific product. |
| GET /api/v1/circularity/{productId}        | Estimates the circularity rate for a specific product.                |
| POST /api/v1/re-indicator:                 | Submits data to calculate a new RE indicator.                         |
| POST /api/v1/disassembly-effort            | Submits data to calculate new disassembly efforts.                    |
| POST /api/v1/circularity                   | Submits data to calculate a new circularity rate.                     |

#### RE-indicator calculation

This component is responsible for calculating the RE indicators. It
fetches the necessary DPP data from the DPP I/O service and performs the
calculations. It interacts with the RE database to store and retrieve RE
indicator data and utilizes DPP information for accurate calculations.

#### Disassembly effort 

The Disassembly Effort component calculates the efforts required to
disassemble a product. This includes assessing the time, tools, and
complexity involved in taking apart the product for recycling or repair.
It uses DPP data and stores the results in the RE database. It ensures
that the disassembly process is evaluated based on up-to-date product
information.

#### Circularity

The Circularity component calculates indicator(s) reflecting the degree
of circularity at product level. Specific indicators are listed in
CE-RISE Deliverable 2.1. It integrates DPP data to determine the
circularity metrics and stores the results in the RE database.

#### RE-database

The RE Database stores data related to the RE indicators, disassembly
efforts, and circularity calculations. It acts as a central repository
for all the calculated metrics and linked to the products scanned by
each user. It supports the storage and retrieval of data for the
components comprising the RE-service components, ensuring persistence
and accessibility of the calculated metrics.

### SEE Service (white box)

Figure 8 outlines the components of the SEE Service, which constructs
lifecycle inventory (LCI) models, and computes life cycle impact
assessment calculations as input to sustainability impact assessments.
Key components include the LCI Generator, SEE Calculator, and System
Matcher, all coordinated by the API component. The SEE Service gathers
data from various sources, including LCI and SUT databases, and
integrates it with information sourced from the Digital Product Passport
(DPP). The API Gateway facilitates external requests, routing them to
the SEE Service for processing.

<img src="media/fig8.png" style="width:6.38889in;height:5.42361in" />

<span id="_Ref170226907" class="anchor"></span>Figure 8: Component
overview of SEE service container

#### SEE service API

<span id="_Toc170654513" class="anchor"></span>Table 19: Example API
endpoints for the SEE service.

| **Endpoint**                    | **Returns**                                             |
|---------------------------------|---------------------------------------------------------|
| GET api/v1/all\_impacts/{id }   | Vector of all impact indicator results                  |
| GET api/v1/RE\_impacts/{id}     | Impact indicators associated with a RE strategy         |
| GET api/v1/contributions/{id}   | Contribution analysis result                            |
| GET api/v1/env\_impacts/{id}    | Returns environmental impact indicators (subset of all) |
| GET api/v1/social\_impact/{id } | Only returns social impact indicators (subset of all)   |

#### LCI generator 

Python package that constructs a square system representing an LCI out
of the LCI database, SUT database, foreground model following from the
DPP chain of custody. Note that this component includes a gap-filling
routine for which the approach and decision is under development (in
WP3).

#### Sector classifier

Ensures that the foreground model obtained from the DPP can be linked to
the background processes and sectors available in the Life Cycle
databases. This is done by classification of the individual LCI elements
from the DPP to the sectors or product groups available in LCI and SUT
databases. For example, the field “electricity use” in the DPP needs to
correspond to the relevant electricity production mix as specified in
the background tables.

#### SEE calculator

Suite of standard LCA calculation routines that allow for the
calculation of environmental and social economic footprints, as well as
functionality such as contribution analysis.

### Product Information (white box)

The API Gateway routes external requests to the Product Information
Service container. Within the Product Information Service, the API
component processes requests and coordinates with the Product
Information Processor and Conformity components. The Product Information
Processor extracts product data from the DPP I/O container. It can
combine this with outputs from the RE and SEE service and store results
in the Product database. The Conformity component checks product
information against conformity criteria. The API component returns the
processed results to the API Gateway, which then delivers them to the
requesting client.

<img src="media/fig9.png" style="width:4.4375in;height:3.39935in" />

<span id="_Toc170654528" class="anchor"></span>Figure 9: Component
diagram of key components for the Product information service

#### API

<span id="_Toc170654514" class="anchor"></span>Table 20: Example API
endpoints for the product information service

| **Endpoint**                                | **Returns**                                             |
|---------------------------------------------|---------------------------------------------------------|
| GET /api/v1/product-info/{productId}:       | Fetches product information for a specific product.     |
| POST /api/v1/product-info:                  | Submits data to retrieve or update product information. |
| GET /api/v1/conformity-results/{productId}: | Fetches conformity results for a specific product.      |
| POST /api/v1/conformity-check               | Submits data to perform conformity checks.              |

#### Product information processor

The Product Information Processor component extracts the requested
information about products from the DPP, insofar as this information is
not provided by the RE and SEE services. It interacts with the DPP I/O
container to fetch the Digital Product Passport (DPP) data and stores
the results in the Product Database.

#### Conformity

The Conformity component implements routines to check the conformity of
the product DPP against CE-RISE internal conformity scheme.

### Semantic Search (white box)

Figure 10 presents the Search Service components, designed to handle
complex search queries across various databases. The main components
include the API, Search Engine, LLM (Large Language Model), and RDF/OWL,
all working together to process and enhance search results. The service
interacts with the Product and LCI databases to retrieve relevant data.
The API Gateway routes user queries to the Search Service, where they
are interpreted, executed, and refined before returning the results.

Semantic search involves context awareness of search queries and the
content of documents to provide search outcomes that are more pertinent
and precise. The selection between Large Language Models (LLMs) and
OWL-based knowledge databases for semantic search hinges on specific
application requirements. LLMs, exemplified by GPT models, excel in
understanding natural language and offering context-aware search
results. However, they may struggle with structured knowledge
representation and demand substantial computational resources.
Conversely, OWL-based systems provide precise semantic modeling and
robust reasoning capabilities but require expertise in ontology
development and querying. For applications prioritizing flexibility and
natural language understanding, LLMs may be preferred, while OWL-based
approaches suit scenarios demanding structured knowledge representation
and reasoning capabilities. Hybrid approaches may offer a balanced
solution, combining the strengths of both methods for enhanced semantic
search efficiency and accuracy. There, here we follow the hybrid
approach to achieve the best possible results.

Qdrant, an LLM based open source Vector database tool for semantic
searching, facilitates semantic search by representing data as
high-dimensional vectors, enabling efficient similarity search based on
vector distance metrics (Qdrant, 2024) . By leveraging vector
representations, semantic relationships between data items can be
captured and queried effectively, enhancing the relevance and accuracy
of search results.

<img src="media/fig10.png" style="width:4.68056in;height:4.10566in" />

<span id="_Ref170227038" class="anchor"></span>Figure 10: Component
diagram of the Search service

#### API

<span id="_Toc170654515" class="anchor"></span>Table 21: Example API
endpoints for the semantic search service

| **Endpoint**                 | **Returns**                                                     |
|------------------------------|-----------------------------------------------------------------|
| GET /api/v1/search           | Executes a search query and returns the results.                |
| POST /api/v1/search          | Submits a search query for processing and retrieval of results. |
| POST /api/v1/search/advanced | Submits an advanced search query with more complex parameters.  |

#### Search engine

The Search Engine component processes the search queries forwarded by
the API. It interprets and executes the queries to retrieve relevant
information from the connected databases. It ensures efficient and
accurate retrieval of information by querying the appropriate databases
and integrating the results.

#### LLM

The LLM (Large Language Model) component uses a fine-tuned language
model to interpret and enhance the search queries. It assists in
understanding complex queries and provides more accurate search results.
It processes natural language queries to extract intent and context,
enabling more effective search operations.

#### RDF/OWL

The RDF/OWL (Resource Description Framework / Web Ontology Language)
component provides semantic data representation and reasoning
capabilities. It uses ontologies to improve the accuracy and relevance
of search results. It enhances the search process by linking and
reasoning over structured data, enabling more intelligent and contextual
query responses. RDF is a necessary step in “translating” the
information available in the main Information System databases into a
vector database allowing for semantic searches.

### Databases Module (white box)

The CE-RISE information system contains potentially many database
solutions (relational and non-relational), as is clear from for the
overarching Figures. Rather than detailing the internal structure of
these data storages here, we provide a white box description of the
expected components in a general database structure. Figure 11 provides
the database connection of *any* Information service via a dedicated
connection manager, which handles connection from these services. The
database engine subsequently is responsible for retrieving the data and
managing data storage. To ensure quality of the Information System (see
e.g. Chapter 11) we enable monitoring and logging capability as well as
backup and recovery.

<img src="media/fig11.png" style="width:3.20048in;height:3.68056in" />

<span id="_Ref170227295" class="anchor"></span>Figure 11: Component
overview of generic database container

#### Connection manager

The Connection Manager handles incoming connections from clients and
applications. It manages the establishment, maintenance, and termination
of database connections, ensuring secure and efficient access to the
database. It thus acts as a gateway for all database interactions,
managing user sessions and connection pooling.

#### Database engine

The Database Engine component is responsible for the core functions of
the database, including data storage, retrieval, and updates. It
processes queries and transactions, ensuring data integrity and
consistency. It manages the physical storage of data, executes queries,
and handles indexing, transaction management, and concurrency control.

#### Monitoring and logging

The Monitoring and Logging component continuously monitors database
performance and logs activities. It tracks performance metrics, detects
anomalies, and records transactions and operations for auditing
purposes. It ensures the health and performance of the database by
providing real-time monitoring and detailed logging of database
activities.

#### Backup and Recovery

Database Backup and Recovery manages the data backup and recovery
processes. This component ensures that data are regularly backed up- In
addition, data can be recovered in case of emergencies, such as data
corruption.

### Other Data Ingestion Routines (white box)

Figure 12 outlines a potential structure for the components involved in
collecting data from external services. The API component acts as the
communication interface, between this container and the other services
of the Information System, receiving and handling requests via the API
Gateway. The API component processes these requests and directs them to
the appropriate internal components here represented as generic other
data source 1 and 2. Each of these components is responsible for
interacting with specific external systems to fetch required data. This
architecture facilitates efficient data retrieval and processing by
coordinating multiple data sources and ensuring that the processed
results are returned to the API Gateway, and subsequently to the
requesting client, in a streamlined manner.

<img src="media/fig12.png" style="width:4.4375in;height:3.23468in" />

<span id="_Ref170229315" class="anchor"></span>Figure 12: Component
overview of the data ingestion service broken down by its most important
components

#### API

<span id="_Toc170654516" class="anchor"></span>Table 22: Example API
endpoints for fetching data from different sources.

| ***Endpoint***             | ***Returns***                |
|----------------------------|------------------------------|
| GET /api/v1/data/source\_1 | Fetches data from source 1   |
| GET /api/v1/data/source\_2 | Retrieves data from source 2 |

#### Generic data source 1 and 2 

The Other data source components handle requests to additional external
data sources accessible through APIs. It manages communication with
these sources and returns the processed data to the API component. It
provides flexibility to integrate with various external data sources,
ensuring that data from multiple sources can be accessed and utilized.

### Web-App/Front-End (white box)

Figure 13 showcases the component diagram for the CE-RISE information
system Web Application. It illustrates the key components that enable
user interaction with the information system. It includes the User
Interface, which provides the primary point of interaction for users,
allowing them to input data and view results. Supporting this are the
Utilities component, which contains helper functions and shared logic
used across the application, and the Components component, which offers
reusable User Interface elements to ensure consistency and efficiency in
the interface design. The Authentication component manages user
authentication processes, and will link to a separate authentication
service, ensuring secure access to the application, while the HTTP
Service component handles the transmission of requests to the API
Gateway, enabling communication with backend services.

\|<img src="media/fig13.png" style="width:5.35652in;height:3.47592in" />

<span id="_Ref170228182" class="anchor"></span>Figure 13: Component
overview of Front-end web application container

#### User interface

The User Interface component provides the primary interaction point for
users. It displays information, collects user input, and facilitates
navigation within the web application. It renders the UI elements and
handles user actions, interacting with other components as needed to
perform tasks. Vue.js is intended to be used for programming the user
interface.

#### Utilities

The Utilities component contains helper functions and shared logic that
are used throughout the front-end web application. It provides reusable
code to support various operations. It supports common functionality
such as data formatting, validation, and other utility operations that
can be leveraged by other components.

#### Components

The Components component includes reusable UI elements, such as buttons
and forms, that can be used across the User Interface. These elements
help in maintaining a consistent look and feel throughout the
application. It provides modular and reusable components such as
buttons, forms, and other UI elements that can be used to build the user
interface efficiently.

#### Authentication

The Authentication component handles the front-end parts of user
authentication. It manages login, logout, and session management,
ensuring that users are properly authenticated before accessing certain
parts of the application. It verifies user credentials, manages user
sessions, and ensures secure access to the web application.

#### HTTP service

The HTTP Service component is responsible for transmitting requests from
the front-end web application to the API Gateway. It handles
communication with backend services. It sends HTTP requests to the API
Gateway and processes the responses, enabling the front-end to interact
with backend services.

### API Gateway (white box)

Figure 14 highlight the individual components of an API Gateway. The
role of the gateway is to act as the central routing mechanism for
incoming requests to various backend services. Key components include
the Controller, Routing Engine, Request Transformation, and Monitoring
and Logging. The Controller receives and parses requests, the Routing
Engine directs them to appropriate services, and the Request
Transformation ensures data consistency. The Monitoring and Logging
component tracks performance and activities, providing insights into the
system's health. This architecture ensures efficient request handling
and communication between front-end applications and services like RE,
SEE, and Product Information services.

<img src="media/fig14.png" style="width:3.01054in;height:4.03478in" />

<span id="_Ref170227727" class="anchor"></span>Figure 14: Component
overview of API gateway

#### API controller

The Controller component listens to incoming requests from the Front-end
Web Application. It acts as the initial point of contact for handling
requests. Function: It receives and parses incoming requests, passing
them to the Routing Engine for further processing.

#### Routing engine

The Routing Engine component matches incoming requests to specific
backend services based on predefined rules and routes. It determines the
appropriate backend service, such as the RE-service, the SEE-service, or
the product information service, to handle each request and forwards the
request to that service.

#### Request transformation

The Request Transformation component modifies requests and responses as
needed. This might include tasks such as formatting data, adding
headers, or transforming payloads. It ensures that requests and
responses are in the correct format and contain the necessary
information for further processing.

#### Monitoring and logging

The Monitoring and Logging component collects and logs activities and
performance metrics for the API Gateway. It provides insights into the
functioning and usage of the gateway. It tracks requests, logs errors,
monitors performance, and generates reports on API usage and health.

# RUNTIME VIEW

In this chapter, we showcase user interaction with the system by
outlining the sequence diagrams. We focus on a subset of the full
collected user stories to outline the selected important features of the
system. For clarity and to decrease the complexity of the sequence
diagrams, we remain focused on the interaction between the front-end web
application, the API gateway routing requests to the different
microservices and the interlinkages between the different services at
the C4-container level. In a similar vein, we have omitted the
authentication and authorization routines from the sequence diagrams
presented in the following sections. More detailed sequence diagrams,
highlighting the flow between C4-components across and within the
individual services, will be made available once implemented as part of
the online code and architecture documentation at
<https://git.nilu.no/ce-rise/architecture>.

The runtime scenarios described are the following. The retrieval of a
specific piece of information from a product DPP (section 7.1). The
calculation of a RE-indicator, represented here by repairability by the
RE module of the CE-RISE information system (section 7.2). The
calculation of a SEE footprints using the SEE module of the CE-RISE
information system and building on a DPP chain-of-custody – a system of
linked DPP representative of the supply and value chains (section 7.3).
Section 7.4 describes the top-level sequence aimed at benchmarking the
difference in SEE impacts between different RE-strategies employed for a
product.

The Information System will contain monitoring capabilities retaining
logs of the products scanned by a given user. Section 7.5 describes how
users can retrieve information and aggregate this information (so-called
roll-ups) for a given period. Section 7.6 describes a natural language
query across the CE-RISE information system databases.

One of the key capabilities following from linked product DPPs is the
enabling of traceability in the supply chain. Particularly raw materials
may be sold as batch products, e.g. a batch of metal ingots, where raw
materials are sourced from different suppliers. The DPP infrastructure
can keep track of these suppliers, and extraction this information out
of the DPP chain-of-custody is described in Section 7.7.

Section 7.8 highlights the case of consumers viewing circular economy
relevant information associated with a product, represented through a
circularity rate. In principle, the user type (in this case, consumer)
does not influence the functioning of the microservices, but we would
like to highlight lighter access management and limited availability of
information and features for consumers, i.e. consumer users will gain
easier access to the system but can only access publicly available DPP
information.

Whereas most scenarios described above start out from reading a DPP or
chain of DPPs as basis for assessment, we focus in Section 7.9 on
describing writing information to the DPP. This is represented by a
repairer describing a repair action for a product and adding this to the
DPP.

It should be noted that, in the following sequence diagrams user
interaction with the system captures various stages. Users are assumed
to be logged in, in addition they are assumed to scan a QR code
representing a unique DPP identifier, or otherwise provide this
identifier as QR codes may not be the ideal representation of a DPP on a
product. Finally, some functionality requires user input, to be given
through web forms in the web interface, or through uploading
documentation. This input is assumed to be transmitted along with the
API request to the API gateway.

## Retrieve Information from a DPP

The sequence diagram in Figure 15 illustrates the process of a user
(Collector) scanning an identifier, such as a QR code, to obtain an
assembly manual from a Digital Product Passport (DPP). The process
begins with the Collector using the Information System front-end to scan
a QR identifier associated with a product. The front-end then sends a
request to the API Gateway to retrieve the assembly manual linked to or
incorporated into the DPP associated with the scanned QR code. The API
Gateway receives the request and routes it to the Product Information
Service, which is responsible for handling such queries. The Product
Information Service forwards a request to the DPP I/O service, asking to
collect the DPP data from the DPP Service Provider. The DPP I/O service
makes an API call to the DPP Service Provider, requesting the specific
DPP data related to the product. The DPP Service Provider processes the
request and returns the DPP data back to the DPP I/O service, which then
sends the retrieved DPP data back to the Product Information Service.
The Product Information Service processes the received DPP data to
extract the required assembly manual. Once the assembly manual is
processed, it is returned to the API Gateway by the Product Information
Service. The API Gateway forwards the assembly manual to the Information
System front-end, which then displays the information to the Collector.
Throughout this process, various components interact seamlessly to
retrieve and process the required product information, ensuring that the
Collector receives the assembly manual efficiently. The diagram
emphasizes the coordinated roles linking the API Gateway, Product
Information Service, DPP I/O, and the DPP Service Provider in handling
such requests.

<img src="media/fig15.png" style="width:6.49028in;height:2.08125in" alt="A diagram of a diagram Description automatically generated with medium confidence" />

<span id="_Ref170571628" class="anchor"></span>Figure 15: Sequence
diagram showing simple data retrieval from a DPP hosted at an external
DPP provider.

## Calculate RE-indicators for a Product

The sequence diagram in Figure 16 illustrates the process of a user
(Collector) scanning a QR identifier to obtain the repairability index
of a product from a Digital Product Passport (DPP). The process begins
with the Collector using the Information System to scan the QR
identifier. The Information System front-end sends a request to the API
Gateway to retrieve the repairability index for the product associated
with the scanned QR code. The API Gateway receives the request and
routes it to the RE service, which is responsible for calculating or
retrieving the repairability index. The RE service forwards a request to
the DPP I/O component to collect the relevant DPP data from the DPP
Service Provider. The DPP I/O component makes an API call to the DPP
Service Provider, requesting the specific DPP data related to the
product. The DPP Service Provider processes the request and returns the
DPP data to the DPP I/O component, which then sends the retrieved DPP
data back to the RE service. The RE service processes the DPP data and
calculates the repairability index, if it is not already included in the
DPP. Once the repairability index is determined, it is returned to the
API Gateway by the RE service. The API Gateway forwards the
repairability index to the Information System front-end, which then
displays the information to the Collector. This process highlights the
coordinated interaction between the API Gateway, RE service, DPP I/O,
and the DPP Service Provider to efficiently retrieve and process the
necessary product information, ensuring that the Collector receives the
repairability index promptly.

<img src="media/fig16.png" style="width:6.49792in;height:2.55208in" alt="A diagram of a diagram Description automatically generated" />

<span id="_Ref170591387" class="anchor"></span>Figure 16: Sequence
diagram showing calculation of a Repairability index for a product.

## Calculate SEE-impact for a Product

The sequence diagram in Figure 17 illustrates the process of a
manufacturer requesting a SEE (Socio-economic and environmental) impact
calculation for a product using the Information System. The process
starts with the manufacturer either scanning a QR code or providing an
identifier via the front-end of the Information System. This application
then sends a request to the API Gateway to calculate the SEE impact for
the specified product. The API Gateway routes the request to the SEE
service, which is responsible for performing the impact calculations.
The SEE service initiates the process by requesting the chain of custody
information for the DPP from the DPP I/O service. The DPP I/O service
makes an API call to the DPP Service Provider to retrieve the required
DPP data. The DPP Service Provider processes the request and returns the
DPP data to the DPP I/O service, which then sends the relevant parts of
the DPP and chain of custody information back to the SEE service.

Once the SEE service receives this information, it collects the
necessary background data from the LCI (Lifecycle Inventory) background
database and the SUT (Supply Use Table) background database. The SEE
service then retrieves data from these databases and matches the
foreground data with the background data to generate a comprehensive
LCI. After generating the LCI, the SEE service performs the calculations
required to determine the SEE impact results. These results are then
returned to the API Gateway, which forwards them to the front-end, which
displays the results to the manufacturer. This detailed sequence
highlights the interaction between various components, including the API
Gateway, SEE service, DPP I/O, LCI background database, SUT background
database, and the DPP Service Provider, to ensure the efficient
calculation and delivery of the SEE impact results to the manufacturer.

<img src="media/fig17.png" style="width:6.49444in;height:2.50208in" alt="A screenshot of a computer program Description automatically generated" />

<span id="_Ref170591550" class="anchor"></span>Figure 17: Sequence
diagram showing the calculation of a SEE-impact for a product.

## Comparative SEE Impacts for RE Strategies

The sequence diagram in Figure 18 outlines the process by which a
collector uses the Information System to inquire about the associated
SEE impacts for available RE-strategies pertaining to a product. The
interaction begins with the collector scanning a QR code or providing
the DPP identifier to the system. This application sends a request to
the API Gateway, effectively asking *"What are available RE strategies
for this product and associated SEE impacts?"* The API Gateway then
routes this request to the Product Information Service.

The Product Information Service first requests the available RE
strategies for the product by asking the RE service for the product's
DPP. The RE service requests the DPP from the DPP I/O service, which in
turn makes an API call to the DPP Service Provider. The DPP Service
Provider responds with the DPP data, which is relayed back through the
DPP I/O service to the RE service. The RE service then returns the
available RE strategies to the Product Information Service.

Next, the Product Information Service requests the SEE impacts for the
product by forwarding the request to the SEE service. The SEE service
requests the chain of custody for the DPP from the DPP I/O service,
which again calls the DPP Service Provider and receives the DPP data.
The SEE service collects additional necessary data from the LCI and SUT
background databases. The SEE service matches the foreground data with
the background data to generate a comprehensive LCI and calculates the
SEE impact results associated with each RE strategy.

These results are forwarded to the Product Information System. The
Product Information Service compiles the results and sends them back to
the API Gateway. The API Gateway then forwards the results to the system
front-end, which displays the information to the collector. This
sequence demonstrates the collaborative efforts of various components
including the API Gateway, Product Information Service, RE service, SEE
service, DPP I/O, LCI background database, SUT background database, and
the DPP Service Provider in retrieving and processing data to provide
detailed information on RE strategies and their SEE impacts to the
collector.

<img src="media/fig18.png" style="width:6.3875in;height:2.48542in" />

<span id="_Ref170591558" class="anchor"></span>Figure 18: Sequence
diagram showing the comparative calculation of SEE impacts based on
standard processing schemes

## Roll-Up of All Processed Products by Single Actor

The sequence diagram in Figure 19 illustrates the process by which a
repairer logs into their Information System account requests a report of
all products repaired in the previous month. This request is sent to the
API Gateway, which then routes the request to the Product Information
Service. The Product Information Service processes the request by
querying the product database for all products processed by the specific
repairer within the requested timeframe.

The product database returns a results table containing the relevant
data back to the Product Information Service. The Product Information
Service then sends these results to the API Gateway, which in turn
returns the results to the front-end application. The front-end
application displays the information to the repairer, providing a
summary of all the products repaired. Additionally, the repairer has the
option to download this information as a CSV file for further analysis
or record-keeping. This sequence highlights the collaborative
interactions between the front-end application, API Gateway, Product
Information Service, and the product database, ensuring that the
repairer can efficiently access and download a detailed summary of their
repair activities for the previous month or any other given time period.

<img src="media/fig19.png" style="width:6.48681in;height:2.11181in" alt="A diagram of a diagram Description automatically generated" />

<span id="_Ref170591714" class="anchor"></span>Figure 19: Sequence
diagram showing the history of all processed products by a repairer in
the last month

## Semantic Search Workflow

The sequence diagram in Figure 20 illustrates the process of a user
entering a search query in natural language in the Information System.
The process begins with the user inputting their search query into the
front-end application, which then transmits this query to the API
Gateway. The API Gateway routes the request to the Search Service, which
is responsible for processing the query. The Search Service forwards the
query to the search model, which, in this scenario, conducts searches in
both the LCI and Product Database. These databases return the search
results to the search model, which then processes and matches the search
results. The processed results are sent back to the Search Service,
which in turn sends them to the API Gateway. The API Gateway returns the
results to the front-end application, where the user can view the
displayed search results. This sequence showcases the interaction
between the user, front-end application, API Gateway, Search Service,
LCI Database, and Product Database to handle and respond to a search
query efficiently. The process ensures that the user's query is
interpreted, relevant data is retrieved from multiple sources, and the
results are processed and returned in a user-friendly format.

<img src="media/fig20.png" style="width:6.48681in;height:2.64375in" alt="A screenshot of a computer program Description automatically generated" />

<span id="_Ref170591802" class="anchor"></span>Figure 20: Sequence
diagram of searching through LCI and Product databases using natural
language queries

## Traceability of Secondary Raw Materials

The sequence diagram in Figure 21 shows a material supplier uses the
Information System to obtain the location of origin for materials in a
specific batch by scanning a the DPP. The interaction begins with the
material supplier scanning the DPP identifier of the material batch. The
information system front-end sends a request to the API Gateway, seeking
the location of origin for the materials. The API Gateway routes this
request to the Product Information Service. The Product Information
Service, upon receiving the request, asks the DPP I/O service to collect
the necessary Digital Product Passport (DPP) data from the DPP Service
Provider.

The DPP I/O service makes an API call to the DPP Service Provider to
obtain the DPP data and the chain of custody information for the
materials. The DPP Service Provider processes the request and returns
the required DPP data to the DPP I/O service, which then sends this
information back to the Product Information Service. The Product
Information Service extracts the raw material origins from the DPP data
and compiles the necessary information. Once this process is complete,
the Product Information Service returns the result to the API Gateway.

The API Gateway, in turn, sends the result back to the front-end
application. The front-end application then displays the information
about the location of origin for the materials to the material supplier.
This sequence highlights the coordinated interaction between various
components, including the API Gateway, Product Information Service, DPP
I/O, and the DPP Service Provider, ensuring that the material supplier
can efficiently access and obtain detailed information about the origins
of the materials in a specific batch.

<img src="media/fig21.png" style="width:6.48681in;height:2.01736in" alt="A white background with text boxes Description automatically generated with medium confidence" />

<span id="_Ref170591880" class="anchor"></span>Figure 21: Sequence
diagram representing the tracing of raw materials from the
chain-of-custody contained in the DPP.

## Consumer Views CE Information

The sequence diagram in

Figure 22 depicts the process of a consumer using the Information System
to obtain Circular Economy information for a product by scanning a QR
identifier. The consumer initiates the process by scanning the QR code
with the front-end application, which then sends a request for Circular
Economy information to the API Gateway. The API Gateway routes this
request to the RE service, which is responsible for processing the
request. The RE service forwards a request to the DPP I/O service to
collect the necessary Digital Product Passport (DPP) data from the DPP
Service Provider. The DPP I/O service makes an API call to the DPP
Service Provider, requesting the specific DPP data for the product. The
DPP Service Provider processes the request and returns the DPP data to
the DPP I/O service, which then sends the retrieved DPP data back to the
RE service.

Upon receiving the DPP data, the RE service computes, for example, the
circularity rate for the product. The results are returned by the RE
service to the API Gateway. The API Gateway then forwards the processed
result back to the Information System front-end, which displays it to
the consumer.

<img src="media/fig22.png" style="width:6.49028in;height:2.575in" alt="A screenshot of a computer screen Description automatically generated" /><span
id="_Ref170591979" class="anchor"></span>

Figure 22: Sequence diagrams of consumer requesting and viewing a
product circularity rate.

## Write to DPP Through Information System

The sequence diagram in

Figure 23 illustrates the process by which a repairer updates the DPP
with information about repair actions taken. The process begins with the
repairer scanning the DPP (QR) code and describing the repair action
taken using a front-end application. The front-end application sends a
request to add this information to the DPP to the API Gateway. The API
Gateway routes the request to the Product Information Service, which
then requests to write the repair information to the DPP at the DPP
Service Provider.

The Product Information Service forwards this request to the DPP I/O
service, which makes an API call to the DPP Service Provider to update
the DPP with the new repair information. The DPP Service Provider
processes this request and confirms the successful update of the DPP.
This confirmation is sent back to the DPP I/O service, which then
forwards it to the Product Information Service.

Upon receiving the confirmation, the Product Information Service logs
the repair action into the product database under the repairer's
account. Finally, the Product Information Service returns the
confirmation to the API Gateway, which then sends it back to the
front-end application. The front-end application displays the
confirmation to the repairer, indicating that the repair action has been
successfully logged and added to the DPP.

<img src="media/fig23.png" style="width:6.19669in;height:1.59722in" alt="A diagram of a computer Description automatically generated with medium confidence" /><span
id="_Ref170592113" class="anchor"></span>

Figure 23: Sequence diagram of a repairer updating the DPP of a product
with information about the repair.

# DEPLOYMENT VIEW

## Deployment During the CE-RISE Project

We intend to deploy the system using resources located on NILU's
premises. NILU operates Linux servers capable of running Docker
containers, which are essential for the CE-RISE information system. Each
microservice is managed in its own GitLab repository, containing the
service's code, Dockerfile, and a docker-compose file for local
development. The CI/CD pipeline for each microservice includes stages
for building, testing, and deploying the Docker images. These images are
stored in the GitLab Container Registry. A main docker-compose file,
stored in a deployment repository, can be used to spin up all containers
required for the system. This file includes the configuration for all
microservices and database containers, ensuring consistent deployment
across testing and production environments.

Using GitLab CI/CD, we automate the build, test, and deployment
processes. When updates are made to a microservice, the pipeline
triggers the deployment repository, which uses the latest Docker images
to update the running application. This setup ensures an efficient,
modular, and scalable deployment process for the CE-RISE information
system. With a small team our approach focuses on agility and rapid
iteration.

We use a development (dev) environment for continuous development and
testing directly within Docker containers, closely mirroring production.
This setup minimizes differences between development and deployment
scenarios. Our Production (prod) environment is where our applications
are deployed for end-users. By concentrating on these two
environments—dev for active development and testing, and prod for live
deployment—we maintain an effective workflow. If considered appropriate,
testing may also be performed in a separate testing environment. This
approach lets us prioritize speed and responsiveness while ensuring our
applications meet user needs efficiently. NILU provides versatile
deployment capabilities with KVMs available for Linux systems, offering
flexible configurations of CPUs and RAM.

## Long Run Production System

In the long run, our vision for the production system involves a
strategic deployment model. To ensure the highest levels of data
protection, the Authorization Manager and the Input Data Retrieval
Engine will remain on NILU's premises. These components play a critical
role in safeguarding (private) data and managing user access.

In contrast, the remaining building blocks of the infrastructure could
be transitioned to more cost-effective and performance-enhancing cloud
solutions if this is justified by the data flow sizes and costs
associated with maintaining these on premise. This approach allows us to
harness the scalability and efficiency of cloud technologies while
maintaining stringent data security measures where needed. It ensures a
balance between performance, cost-effectiveness, and data protection in
the long-term operation of the system.

# CROSS-CUTTING CONCEPTS

In this Chapter, we describe concepts that are relevant to multiple
parts of the CE-RISE information system. These concepts form the basis
for the conceptual integrity of the architecture (Hruschka and Starke,
2023) and are aimed at providing internal consistency.

## Communication and Integration

The individual Information System microservices will communicate using
RESTful APIs. This approach ensures that services can communicate using
standard HTTP methods, providing a straightforward and widely adopted
method for inter-service communication. All microservices will be
deployed as Docker containers and run on the same Docker container
network, ensuring seamless interaction and integration. This setup
simplifies networking configuration and management, allowing containers
to communicate using container names rather than IP addresses. Each
microservice will be assigned a unique port for its API endpoints.
Proper port allocation and management will be crucial to avoid conflicts
and ensure reliable communication. This will be documented and managed
as part of our deployment process. A dedicated KeyCloak container will
handle authentication and authorization across all microservices.
KeyCloak will manage user sessions and token-based authentication,
ensuring secure access to the APIs. Microservices will verify user
tokens with KeyCloak before processing requests.

## Session Handling, Logging and Tracing

We will use Role-Based Access Control (RBAC) to manage user permissions
and ensure secure access to the system. KeyCloak, an open-source
identity and access management solution, will be our primary tool for
implementing RBAC. KeyCloak supports fine-grained authorization
policies, making it ideal for managing user roles and permissions
efficiently. This ensures that only authorized users can access specific
functionalities and data, enhancing security and operational integrity.

**Logging and monitoring:**

To complement our RBAC implementation, we will establish a comprehensive
logging and monitoring framework. Logging is crucial for tracking system
activities, detecting unauthorized access attempts, and ensuring
compliance with security policies. Our logging strategy will include:

-   **Access Logs**: Recording all access attempts, including successful
    and failed login attempts, and resource access requests.

-   **Audit Logs**: Capturing detailed records of user activities, such
    as data modifications and administrative actions.

-   **Security Logs**: Monitoring for suspicious activities, such as
    multiple failed login attempts and unauthorized access attempts.

The combination of RBAC, KeyCloak, and logging forms a comprehensive
security framework for the CE-RISE system. KeyCloak manages user
authentication and authorization, while logging provides a detailed
record of all user activities and system events. This integration
ensures:

-   **Security**: Only authorized users have access to sensitive data
    and functionalities

-   **Accountability**: Detailed logs provide a clear audit trail of
    user actions.

-   **Compliance**: Logging and monitoring help meet regulatory
    requirements and ensure consistent application of security policies.

## Containerization of Building Blocks

Containerization is a fundamental architectural concept where each
building block of the system is encapsulated within its own
self-sufficient container. This modular approach ensures that components
can operate independently and consistently, simplifying deployment and
scalability. Containers provide isolation and portability, making it
easier to manage and orchestrate individual building blocks within the
overall system architecture.

We prioritize the organization of microservices in a manner that
promotes reusability and interoperability with external systems. Each
microservice is designed to be self-contained and isolated, capable of
functioning independently without reliance on other services. By
adhering to this architectural principle, we ensure that external
parties can seamlessly integrate and reuse individual microservices in
their own systems, maximizing their utility and versatility.

In order to maintain consistency and streamline deployment processes,
all Docker containers within the CE-RISE system will be configured in a
similar fashion. This approach ensures that each container is
provisioned and managed in a similar manner, promoting ease of
maintenance, scalability, and interoperability across the entire system.
By adopting a uniform setup across all containers, we aim to minimize
configuration discrepancies and simplify the management of the
containerized infrastructure.

## Code Documentation

We will adhere to the principle of self-documenting code, which
emphasizes using clear, descriptive variable and function names instead
of writing comments within the code. This approach ensures that the code
itself is easy to read and understand, reducing the need for additional
explanatory comments. By naming variables and functions meaningfully, we
make the codebase more maintainable and comprehensible to all team
members, promoting better collaboration and reducing the risk of
misinterpretation. This practice aligns with the widely accepted clean
code conventions and helps maintain a high standard of code quality
throughout the project.

Each microservice in GitLab will be documented with its own README.md
file, describing the basic actions to set up and use that specific
microservice, as well as managing different environments like testing or
production. The README will contain important details such as keys,
versions, and tokens. Focusing on documentation increases the
understandability and use of individual microservices.

Furthermore, each microservice API will be documented utilizing a
standard protocol and software for API documentation (e.g., OpenAPI,
Swagger, Postman). This ensures a comprehensive understanding of the
functionalities offered, and it facilitates the ease of extension and
reuse of the services provided.

Finally, we maintain an updated version of the architecture, reflecting
the developed code here: <https://git.nilu.no/ce-rise/architecture>

## UI, UX, and Accessibility

We recognize the importance of User Interface (UI) and User Experience
(UX) design, as well as accessibility considerations, in creating an
effective and inclusive system. We are committed to addressing UI, UX,
and accessibility requirements to the best of our abilities. We plan to
incorporate basic principles of usability and accessibility in our
design processes and will seek external guidance and resources as
needed. Our approach will be to continuously improve the user interface
and experience based on feedback from users and stakeholders.

Accessibility of model components will be ensured by the adoption of
standard approaches to document API specifications, as described above.
The interactive documentation provided by tools such as Swagger UI[6] or
Redoc[7] will be exploited to let developers experiment with the API,
see real-time responses, and better understand how the API behaves with
actual data.

## Version Control and Branch Management

All source code will be hosted in dedicated Git repositories using the
version control made available by the system hosted at NILU. We're
aiming for clear and consistent commit messages practice. For example,
by preventing the mixing of different types of changes in single code
commits. This ensures a commit history that is clean and easy to follow,
so everyone can see how our code has evolved over time.

We will manage the Git repository creating a dedicated branch for each
new issue, feature, or significant update. This approach, often referred
to as feature branching, helps maintaining the stability of the main
branch while allowing developers the freedom to experiment and develop
independently. Each new feature or issue is developed on its own branch,
which is derived from the latest stable version of the main branch. This
isolation prevents disruptions in the main codebase and facilitates
parallel development across multiple features or fixes.

Before a feature branch is merged back into the main branch, it will
undergo a series of checks and reviews to ensure quality and
compatibility. This includes code reviews, automated testing, and
integration testing to confirm that the new changes do not adversely
affect existing functionality. This will promote project standards but
also store a clean and functional history of changes, making it easier
to track modifications and resolve potential issues in future
development.

# ARCHITECTURAL DECISIONS

This document presents the initial architecture of the CE-RISE
Information System. It is written before implementation of the system
and serves as a guide for implementation. Throughout this document, some
key architectural decisions regarding the system were made. Insofar as
these are not already highlighted in the previous Chapters, we aim here
to motivate the main architectural decision (section 10.1) as well as
highlight alternative choices considered not relevant or out of scope,
for the CE-RISE project implementation (section 10.2).

##  Main Architectural Decisions

### Microservice Architecture

In terms of architecture, the CE-RISE Information System application
will be built using a **Microservices** structure approach (i.e., a
collection of smaller, independently deployable services), as opposed to
the traditional monolithic structure approach. Operating with a single
front-end interface, will manage and support a separate part of the
platform, dealing with specific workloads (i.e., business capabilities).

The adoption of this structure is motivated by several factors including
the ability to update, modify, deploy or scale each service as required,
given they run independently from one another.

The CE-RISE IS application will comprise the following services
developed within the project:

1.  DPP I/O

2.  RE service

3.  SEE service

4.  Product information service

5.  Semantic search service

6.  Data ingestion service

7.  API Gateway

Supported by the following databases: LCI database, SUT database,
Product database, RE database, and a Search (vector) database. In
addition, the system will use existing code for authorization and
authentication infrastructure.

These services are coupled and will communicate among them over a
network, using HTTP and messaging queue lightweight protocols, thus
enabling inter-service communication. Each service is responsible for
enabling a set of features as described in Chapters 6 and 7 of the
application. Through the envisioned API gateway, the full set of
features will be made available to the front-end, while ensuring that
management of the different API requests is kept separate from the
front-end. This choice also enables an opportunity to make available the
Information System features programmatically, i.e. via a network link to
other software, thus facilitating future interoperability with external
systems.

The choice of the microservices architecture is meant to ensure that the
development of the proposed solution is done in a fluid, comprehensive
and consistent way, allowing for further development – and in a
frictionless way – if deemed necessary. The following advantages from
the adoption of microservices architecture presents benefits for the
CE-RISE IS application:

**Agility -** CE-RISE IS will benefit from development, test,
deployment, and scaling of each service being done independently by
different developers.

**Scalability -** CE-RISE IS will benefit from its services to be able
to support distributed systems with services being scaled on demand – to
handle more traffic loads – and independently of one another.

**Resilience -** Failure in one service will not necessarily impact the
well-functioning of the entire CE-RISE IS application.

**Ease of maintenance -** CE-RISE IS application will benefit from high
maintainability of the microservices, making it easy to debug and fix
failures.

**Diversity of technology -** Services within CE-RISE IS application
will be built using desired technologies, databases, and frameworks,
based on the specified requirements, providing room for continuous
innovation and improvement.

Disadvantages of the microservices architectural approach lie primarily
in defining the complexity of orchestrating the inter-service
communication, potential issues with latency, and increased resource
consumption as well as operational overhead. However, these
disadvantages were found to be less relevant in the context of CE-RISE
than those of a monolithic architecture.

### Use of Available Resources 

Our architecture decisions have been influenced by available resources
and by the foreseeable path forward on development and deployment. As
mentioned above, initially we envision a system running on the premises
and based on technologies in the current stack.

As indicated in Chapter 8.1, the deployment of the CE-RISE information
system will occur at NILUs internal servers. This choice is motivated in
part by the availability of this infrastructure as part of the
technology stack, the experience of the development team, and support
resources available. Added considerations are related to compliance with
data protection regulation and security, as described in section 10.2.2.
The need to scale up the system will be assessed iteratively and may
determine the opportunity to migrate parts of the system to commercial
cloud solutions, as supported by the adoption of a microservice-based
structure.

The lead developing teams at NILU and University of Oslo have a
sufficiently rich technology stack and operational know-how across the
full spectrum of software development and deployment (from database
technology, through scientific computing, to development of interactive
web-apps and visualization) to deliver the infrastructure outlined in
this document.

##  Approaches Not Used for the CE-RISE Information System

### Container Orchestration

The architecture presented in this document is considered a compromise
between engineering complexity, overhead, and quality and ambitions
levels during the project implementation. For example, while the
microservices are intended to run in Docker containers, we will not use
container orchestration, such as for example allowed through Kubernetes
or Docker Swam.

While orchestration offers benefits such as automated load balancing and
fault tolerance, these come with added complexity and costs. Given the
modest scale and transaction volume anticipated initially, the system
can be effectively managed with simpler, manual container management
techniques. As the system scales, we can reassess the need for
orchestration to match our evolving requirements.

### Deployment on Commercial Cloud Environments

As indicated in section 10.1.2 above, we will not use deployment on
commercial cloud environments, at least initially. Our server location
choice depends on:

**Data Protection Compliance**: Hosting servers within the EU ensures
adherence to GDPR standards, avoiding the legal uncertainties of
transferring data to non-EU countries, particularly the US.

**Enhanced Trust and User Confidence**: Keeping data within the EU
enhances user trust and demonstrates our commitment to privacy and data
security.

**Operational Efficiency and Futureproofing**: Utilizing EU-based cloud
providers for non-sensitive components offers cloud benefits while
complying with EU regulations, ensuring long-term stability and
mitigating risks associated with potential regulatory changes.

In summary, by keeping critical components on NILU's premises and
leveraging EU-based cloud solutions for other parts, we comply with
regulatory requirements on data protection, enhance user trust, and
optimize performance and cost-effectiveness.

# QUALITY REQUIREMENTS

The eight overall quality goals presented at the beginning in the
Quality Goals presented in section 2.5 can be further decomposed and
extended in 15 detailed requirements. Table 23 lists the extended
quality categories and corresponding requirements.

Quality categories are further developed using quality scenarios,
classified as either usage scenarios or change scenarios. The quality
scenarios can be mapped to the quality categories and visualized as a
quality tree linking scenario to category. The quality tree is presented
in Section 11.1 and quality scenarios are described in Section 11.2.

<span id="_Ref169541209" class="anchor"></span>Table 23: Extended
quality requirements to the CE-RISE information system.

<table>
<thead>
<tr class="header">
<th><strong>Quality Category</strong></th>
<th><strong>Requirement</strong></th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td><ol type="1">
<li><p>Scientific Soundness</p></li>
</ol></td>
<td>Ensure that the system's data, algorithms, and methodologies adhere to scientific principles. Particularly, scientific soundness is relevant for the SEE and RE services.</td>
</tr>
<tr class="even">
<td><ol start="2" type="1">
<li><p>Response Time</p></li>
</ol></td>
<td>Maintain optimal system responsiveness to user queries and interactions by minimizing latency and adhering to predefined response time targets. Response time is bound to the data flows processed by the Information System and does not include the DPP extraction phase external to it.</td>
</tr>
<tr class="odd">
<td><ol start="3" type="1">
<li><p>Proper Management of Authorization and Data Privacy</p></li>
</ol></td>
<td>Implement robust mechanisms for user authentication, access control, and data encryption to safeguard sensitive information and comply with privacy regulations.</td>
</tr>
<tr class="even">
<td><ol start="4" type="1">
<li><p>Reliability</p></li>
</ol></td>
<td>The system should operate reliably without frequent outages or disruptions. High availability and fault tolerance mechanisms may be required to achieve this.</td>
</tr>
<tr class="odd">
<td><ol start="5" type="1">
<li><p>Data Quality</p></li>
</ol></td>
<td>Maintain high standards of data accuracy, completeness, and consistency. Particularly, data quality impacts key services such as the SEE and RE services.</td>
</tr>
<tr class="even">
<td><ol start="6" type="1">
<li><p>Usability</p></li>
</ol></td>
<td>Focus on user-friendly interfaces and intuitive user experiences to make the system accessible to a wide range of users. Conduct usability testing to gather user feedback. Information System usability will be improved through continuous testing in the project pilots.</td>
</tr>
<tr class="odd">
<td><ol start="7" type="1">
<li><p>Security</p></li>
</ol></td>
<td>Protect the system from security threats, including data breaches, unauthorized access, and malicious attacks. Implement robust security measures, encryption, and authentication.</td>
</tr>
<tr class="even">
<td><ol start="8" type="1">
<li><p>Interoperability</p></li>
</ol></td>
<td>Ensure compatibility and seamless integration with external systems, data sources, and APIs. The system should be able to exchange data with other systems efficiently.</td>
</tr>
<tr class="odd">
<td><ol start="9" type="1">
<li><p>Compliance</p></li>
</ol></td>
<td>Adhere to relevant legal and regulatory requirements, such as data protection laws (e.g., GDPR), environmental regulations, and industry standards.</td>
</tr>
<tr class="even">
<td><ol start="10" type="1">
<li><p>Documentation</p></li>
</ol></td>
<td>Maintain comprehensive documentation that covers system architecture, APIs, user manuals, and operational procedures. Well-documented systems are easier to maintain and support.</td>
</tr>
<tr class="odd">
<td><ol start="11" type="1">
<li><p>Performance</p></li>
</ol></td>
<td>Besides response time, consider other performance metrics like throughput, latency, and resource utilization. Optimize system performance to meet user expectations. Continuously monitor system data during piloting activities to identify potential bottlenecks in scaling up operations.</td>
</tr>
<tr class="even">
<td><ol start="12" type="1">
<li><p>Maintainability</p></li>
</ol></td>
<td>Ensure that the system is easy to maintain and update. This includes modular design, version control, and clear code documentation. It will be enhanced by the independent workflows allowed by services modularity.</td>
</tr>
<tr class="odd">
<td><ol start="13" type="1">
<li><p>Scalability</p></li>
</ol></td>
<td>Plan for future growth by designing the system to handle increased loads gracefully. Scalability includes horizontal and vertical scaling options. The containerized modular approach adopted enables the possibility to add modules and to balance their load. This ensures that the system can efficiently handle increased workloads and user demands without a significant drop in performance.</td>
</tr>
<tr class="even">
<td><ol start="14" type="1">
<li><p>Monitoring and Analytics</p></li>
</ol></td>
<td>Incorporate monitoring tools and analytics to track system performance, identify bottlenecks, and make data-driven improvements.</td>
</tr>
<tr class="odd">
<td><ol start="15" type="1">
<li><p>Resilience and Disaster Recovery</p></li>
</ol></td>
<td>Prepare for unexpected failures or disasters by having backup and recovery plans in place to minimize downtime.</td>
</tr>
</tbody>
</table>

##  Quality Tree

Detailed quality requirements can be assessed in quality scenarios as
reported here and explained in the following section. Figure 24 shows
the relationship between the quality categories described above and the
quality scenarios described in Section 11.2.

<img src="media/fig24.png" style="width:6.38403in;height:3.42431in" />

<span id="_Ref170574461" class="anchor"></span>Figure 24: The CE-RISE
Information System Quality Tree.

##  Quality Scenarios

The quality categories are further disaggregated in quality scenarios.
An overview of the scenarios is listed and described in Table 24 below.

<span id="_Ref170574638" class="anchor"></span>Table 24: List of quality
scenarios

| **Quality Scenario**                         | **Description**                                                                                                                                                                                                                                                    |
|----------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| Scenario 1: Data Validation                  | When new data is ingested, the system validates it against a predefined schema and rejects or flags any data that does not meet these standards.                                                                                                                   |
| Scenario 2: Peer Review Integration          | The system integrates with a peer review process, ensuring that all scientific data and models undergo validation and approval before being incorporated into the system.                                                                                          |
| Scenario 3: User Query Response              | Upon receiving a user query, the system ensures that it responds within 2 seconds, meeting the response time requirement for user interactions.                                                                                                                    |
| Scenario 4: Batch Processing Optimization    | For batch processing tasks, the system optimizes its response time to complete large-scale computations within predefined timeframes.                                                                                                                              |
| Scenario 5: Secure User Authentication       | When a user attempts to log in, the system verifies their credentials securely and grants access only to authorized users, ensuring proper authorization management.                                                                                               |
| Scenario 6: Role-Based Access Control        | The system implements role-based access control, allowing different user roles to access specific data and functionalities based on their permissions.                                                                                                             |
| Scenario 7: Redundant Server Configuration   | The system is built to support redundant server configurations, minimizing downtime and ensuring continuous service availability. Redundant server configuration will not be implemented within the project timeline and scope.                                    |
| Scenario 8: Automated Failover               | In case of server failures, automated failover mechanisms seamlessly transfer user requests to healthy servers, minimizing service interruptions.                                                                                                                  |
| Scenario 9: Real-Time Data Validation        | Incoming data is subjected to real-time validation checks, and any data failing validation is immediately rejected, ensuring data quality at the source.                                                                                                           |
| Scenario 10: Data Profiling                  | The system conducts data profiling to identify and rectify data inconsistencies, enhancing overall data quality through proactive measures.                                                                                                                        |
| Scenario 11: User Feedback Integration       | The system integrates a feedback mechanism that allows users to provide input on usability issues, which are then systematically addressed in updates.                                                                                                             |
| Scenario 12: Accessibility Testing           | Usability testing includes accessibility checks to guarantee a user-friendly experience for individuals with disabilities.                                                                                                                                         |
| Scenario 13: Regular Vulnerability Scanning  | The system undergoes regular vulnerability scanning to identify and address potential security weaknesses and threats.                                                                                                                                             |
| Scenario 14: Third-Party API Integration     | The system successfully integrates with third-party APIs used by partner organizations, streamlining data exchange and collaboration.                                                                                                                              |
| Scenario 15: Data Mapping and Transformation | To ensure seamless interoperability, the system employs data mapping and transformation processes to align data structures with external systems.                                                                                                                  |
| Scenario 16: Consent Management              | The system includes a consent management module that allows users to grant or revoke consent for specific data processing activities, ensuring compliance with data protection regulations.                                                                        |
| Scenario 17: On-Demand Help Resources        | Users have access to on-demand help resources, including interactive tutorials and user guides, enhancing their understanding of system functionality.                                                                                                             |
| Scenario 18: Developer Documentation         | Comprehensive developer documentation is provided, offering insights into APIs, code structure, and integration guidelines for third-party developers.                                                                                                             |
| Scenario 19: Load Testing                    | The system undergoes rigorous load testing, simulating heavy user traffic to assess performance under peak conditions and optimize resource allocation.                                                                                                            |
| Scenario 20: Latency Monitoring              | Latency is continuously monitored, ensuring that user interactions and data retrieval processes meet acceptable performance thresholds.                                                                                                                            |
| Scenario 21: Automated Testing Suites        | Automated testing suites are employed to validate system updates and changes, reducing the risk of introducing defects during maintenance.                                                                                                                         |
| Scenario 22: Version Control                 | Version control is implemented to manage codebase changes, facilitating collaboration among development teams and providing traceability of modifications.                                                                                                         |
| Scenario 23: Elastic Resource Allocation     | The system dynamically allocates additional computing resources, such as virtual machines or containers, during periods of high user demand. As the user load increases, the system automatically provisions and scales resources to maintain optimal performance. |
| Scenario 24: Load Balancing                  | To evenly distribute incoming requests, the system utilizes load balancing mechanisms that intelligently route traffic across multiple server instances. Load balancing ensures that no single server is overwhelmed, contributing to efficient scalability.       |
| Scenario 25: Comprehensive Log Retention     | Logs are retained for an extended period, allowing historical analysis of system behavior, troubleshooting, and compliance with auditing requirements.                                                                                                             |
| Scenario 26: Backup Automation               | Automated backup procedures are regularly executed, ensuring data integrity and expedited recovery in the event of data loss or system failure.                                                                                                                    |
| Scenario 27: Disaster Recovery Testing       | Periodic disaster recovery testing is performed to validate the effectiveness of recovery plans and procedures in various disaster scenarios.                                                                                                                      |

# TECHNICAL RISKS

In this Chapter we present a list of technical risks to the CE-RISE
information system (Table 25) implementation together with mitigation
measures. Note that we focus on risks that originate external to the
project and do not focus on “internal” risks such as the availability
and quality of data with project members, the maturity of methods
proposed for implementation by other work packages. These latter
“internal” risks are addressed through frequent interaction between the
system developers in WP4 and the conceptual developers in work packages
2 and 3.

<span id="_Ref170593799" class="anchor"></span>Table 25: A list of
identified technical risks together with mitigation measures

<table>
<thead>
<tr class="header">
<th><strong>Technical risk<br />
(up to 3 levels of priority with *)</strong></th>
<th><strong>Mitigation measures</strong></th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td><strong>Security Vulnerabilities***</strong><br />
Potential security flaws, such as inadequate encryption, SQL injection vulnerabilities, or insufficient access controls, pose a significant risk to the confidentiality, integrity, and availability of the system's data and resources.</td>
<td><ul>
<li><blockquote>
<p>Implement regular security assessments, code reviews, and penetration testing to identify and remediate security vulnerabilities proactively.</p>
</blockquote></li>
<li><blockquote>
<p>Additional0ly, enforce secure coding practices, use encryption for sensitive data, and implement robust authentication and access control mechanisms.</p>
</blockquote></li>
</ul></td>
</tr>
<tr class="even">
<td><strong>Scalability Challenges*</strong><br />
Inadequate scalability may lead to performance degradation or system instability under increased workloads or user demands, potentially resulting in degraded user experience or service disruptions.</td>
<td><ul>
<li><blockquote>
<p>Design the system with scalability in mind, utilizing horizontal scaling, load balancing, and caching techniques to distribute workload efficiently across multiple instances.</p>
</blockquote></li>
<li><blockquote>
<p>Conduct performance testing and capacity planning to identify scalability bottlenecks and optimize resource allocation accordingly.</p>
</blockquote></li>
</ul></td>
</tr>
<tr class="odd">
<td><strong>Dependency Management***<br />
</strong>Dependencies on third-party libraries, frameworks, or services may introduce risks related to version compatibility, maintenance, or vendor support, impacting the system's stability, reliability, and long-term sustainability.</td>
<td><ul>
<li><blockquote>
<p>Maintain a comprehensive inventory of dependencies, monitor for security vulnerabilities and updates, and establish a process for timely dependency management, including version control, dependency injection, and automated dependency scanning.</p>
</blockquote></li>
<li><blockquote>
<p>Consider minimizing dependencies or using dependency management tools to mitigate risks associated with third-party dependencies.</p>
</blockquote></li>
</ul></td>
</tr>
<tr class="even">
<td><strong>Technical Debt Accumulation*<br />
</strong>Cumulative technical debt, such as code duplication, outdated technologies, or undocumented features, may hinder future development efforts, increase maintenance costs, and impede the system's ability to evolve and adapt to changing requirements.</td>
<td><ul>
<li><blockquote>
<p>Prioritize refactoring efforts to address technical debt incrementally, allocate dedicated resources for debt reduction, and establish coding standards, code reviews, and code quality metrics to prevent the accumulation of additional technical debt.</p>
</blockquote></li>
<li><blockquote>
<p>Invest in continuous integration and automated code analysis tools to identify and remediate code smells and maintain codebase hygiene.</p>
</blockquote></li>
</ul></td>
</tr>
<tr class="odd">
<td><strong>Data Integrity and Consistency*</strong><br />
Risks associated with data integrity and consistency, such as data corruption, data loss, or inconsistencies between different data sources or components, may compromise the accuracy and reliability of the system's outputs and analyses.</td>
<td><ul>
<li><blockquote>
<p>Implement data validation, error handling, and data reconciliation processes to ensure data integrity and consistency across different data sources and components.</p>
</blockquote></li>
<li><blockquote>
<p>Utilize transactions, constraints, and auditing mechanisms within the database to enforce data integrity constraints and detect anomalies or inconsistencies proactively.</p>
</blockquote></li>
</ul></td>
</tr>   
<tr class="even">
<td><p><strong>Performance Bottlenecks**</strong></p>
<p>Potential performance bottlenecks, such as inefficient algorithms, database queries, slow asynchronous API calls, or resource-intensive processes.</p></td>
<td><ul>
<li><blockquote>
<p>Profile system performance, identify performance bottlenecks using profiling tools and performance monitoring metrics, and optimize critical components, algorithms, and database queries to improve system performance and responsiveness.</p>
</blockquote></li>
<li><blockquote>
<p>Employ caching, indexing, and asynchronous processing techniques to alleviate performance bottlenecks and enhance scalability.</p>
</blockquote></li>
</ul></td>
</tr>
<tr class="odd">
<td><p><strong>Platform and Infrastructure Risks*</strong></p>
<p>Risks related to platform or infrastructure vulnerabilities, such as hardware failures, network outages, may impact system availability, resilience, and disaster recovery capabilities.</p></td>
<td><ul>
<li><blockquote>
<p>Implement redundancy, failover mechanisms, and disaster recovery strategies to mitigate risks associated with platform and infrastructure failures.</p>
</blockquote></li>
<li><blockquote>
<p>Leverage cloud-based services, load balancers, and distributed architectures to enhance system resilience, availability, and fault tolerance.</p>
</blockquote></li>
<li><blockquote>
<p>Additionally, monitor infrastructure health, apply security patches promptly, and maintain backups to mitigate the impact of potential disruptions.</p>
</blockquote></li>
</ul></td>
</tr>
<tr class="even">
<td><p><strong>Lack of Automated Testing**</strong></p>
<p>Insufficient test coverage or reliance on manual testing processes may result in undetected bugs, regressions, or compatibility issues, increasing the risk of software defects and production incidents.</p></td>
<td><ul>
<li><blockquote>
<p>Establish a robust testing strategy encompassing unit tests, integration tests, and end-to-end tests, and automate testing processes using continuous integration and continuous deployment (CI/CD) pipelines.</p>
</blockquote></li>
<li><blockquote>
<p>Invest in testing frameworks, test automation tools, and test coverage analysis to increase test coverage and identify defects early in the development lifecycle.</p>
</blockquote></li>
</ul></td>
</tr>
<tr class="odd">
<td><p><strong>Inadequate Documentation*</strong></p>
<p>Poor documentation practices, such as incomplete or outdated documentation, may hinder knowledge sharing, collaboration, and troubleshooting efforts among development teams, leading to inefficiencies and delays in development and maintenance tasks.</p></td>
<td><ul>
<li><blockquote>
<p>Prioritize documentation efforts, including requirements documentation, design documentation, and technical documentation, and enforce documentation standards and templates to ensure completeness, accuracy, and relevance.</p>
</blockquote></li>
<li><blockquote>
<p>Utilize documentation tools, wikis, and version control systems to facilitate collaborative documentation and knowledge sharing among team members.</p>
</blockquote></li>
</ul></td>
</tr>
<tr class="even">
<td><p><strong>Compliance and Regulatory Risks***</strong></p>
<p>Risks associated with non-compliance with industry regulations, data protection laws (e.g., GDPR), or organizational policies may result in legal liabilities, financial penalties, or reputational damage, necessitating proactive measures to address compliance requirements.</p></td>
<td><ul>
<li><p>Stay informed about relevant industry regulations, compliance standards, and data protection laws, and implement controls, policies, and procedures to ensure compliance with legal and regulatory requirements.</p></li>
<li><p>Conduct regular compliance audits, risk assessments, and privacy impact assessments to identify and address compliance gaps proactively.</p></li>
<li><p>Additionally, provide training and awareness programs to educate employees about their responsibilities and obligations regarding compliance and data privacy.</p></li>
</ul></td>
</tr>
</tbody>
</table>

# CONCLUDING REMARKS

This architecture provides a first overview of the building blocks and
infrastructure required to deliver the CE-RISE Information System
features. By leveraging a modular architecture, the system offers
robust, scalable solutions tailored to the needs of various stakeholders
in the sustainability and circular economy sectors. The CE-RISE digital
infrastructure is a product under development. This architecture
provides a snapshot development guide for services as per June 2024. To
keep track of changing outlines, as well as provide ongoing
documentation of the system, a digital version of this report, as well
as a repository of all diagrams, is stored under version management in
GitLab at the following address:
<https://git.nilu.no/ce-rise/architecture>

Looking ahead, the CE-RISE system's architecture and technology stack
are designed to accommodate future enhancements and integrations. As new
data sources and analytical methods become available, the system is
designed to be able to incorporate these advancements, further enriching
its capabilities and user value. Additionally, the potential for
increased interoperability and the exploitation of the API gateway
present opportunities for expanded use cases and partnerships.

# GLOSSARY

| **Term**              | **Definition**                                                                                                                                                                                                                                                                                 |
|-----------------------|------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|
| API                   | Application Programming Interface                                                                                                                                                                                                                                                              |
| Arc42                 | Software architecture documentation specification                                                                                                                                                                                                                                              |
| Back-end              | server-side of a software application, responsible for managing and storing data, ensuring security, and handling business logic.                                                                                                                                                              |
| C4                    | Context, Containers, Components, and Code: A software architecture model for visualizing the design of software systems.                                                                                                                                                                       |
| CE                    | Circular Economy                                                                                                                                                                                                                                                                               |
| CI/CD                 | Continuous Integration / Continuous Deployment                                                                                                                                                                                                                                                 |
| Cloud                 | Commercial cloud service such as provided by Amazon Web Services                                                                                                                                                                                                                               |
| Component (C4)        | A part of the C4 model that focuses on how the system is made up of interacting components, defining their responsibilities and the relationships between them.                                                                                                                                |
| Container (C4)        | A part of the C4 model that represents the execution environment for the code, such as an application server, database server, or web browser.                                                                                                                                                 |
| Container (Docker)    | A (Docker) Container includes all components needed to run software, such as the code, libraries, and settings.                                                                                                                                                                                |
| Context (C4)          | A part of the C4 model that provides a high-level overview of the system, showing how it interacts with users and other systems.                                                                                                                                                               |
| Docker                | A platform that uses operating system level virtualization to deliver software in packages called containers.                                                                                                                                                                                  |
| Deployment diagram    | A diagram that shows the physical deployment of artifacts on nodes, such as how software components are deployed on hardware devices.                                                                                                                                                          |
| DPP                   | Digital Product Passport                                                                                                                                                                                                                                                                       |
| Front-end             | Client-side of the web application, responsible for the user interface and user experience, including everything that users will interact with such as web pages, buttons, and forms.                                                                                                          |
| GDPR                  | General Data Protection Regulation                                                                                                                                                                                                                                                             |
| Git                   | Version management and control software                                                                                                                                                                                                                                                        |
| Keycloak              | Authorization and Authentication software                                                                                                                                                                                                                                                      |
| Kubernetes            | An open-source container orchestration platform for automating deployment, scaling, and operations of application containers across clusters of hosts.                                                                                                                                         |
| LCA                   | Life Cycle Assessment                                                                                                                                                                                                                                                                          |
| Middleware            | Software layer that provides essential services and capabilities to the microservices, enabling them to function cohesively as part of a larger application.                                                                                                                                   |
| Module                | A conceptual module as defined in Work Package 2                                                                                                                                                                                                                                               |
| PEF                   | Product Environmental Footprint                                                                                                                                                                                                                                                                |
| RE                    | Repair, Reuse, Repurpose, Remanufacture, Recycle, Refurbish,                                                                                                                                                                                                                                   |
| RE-Criteria           | A set of RE-indicators combined with thresholds that define CE-RISE conform standards for RE-indicators. When these criteria are met, the product is considered CE-RISE conform and therefore expected to contribute significantly to advancing the Circular Economy and reducing SEE impacts. |
| RE-indicator          | A quantitative measure of the applicability of a RE-strategy to a product, calculated by the CE-RISE system from RE-parameters.                                                                                                                                                                |
| RE-parameter          | A property of a product, component, or material (system) that is used to calculate RE-indicators.                                                                                                                                                                                              |
| REST                  | Representative State Transfer                                                                                                                                                                                                                                                                  |
| SEE                   | Socio-environmental economic                                                                                                                                                                                                                                                                   |
| SEE – impact          | Social environmental Economic impact indicator                                                                                                                                                                                                                                                 |
| SEMVER                | Semantic Versioning                                                                                                                                                                                                                                                                            |
| Service/ microservice | A small, autonomous service that works together with other microservices to form a complete application.                                                                                                                                                                                       |
| SOA                   | Service Oriented Architecture                                                                                                                                                                                                                                                                  |
| TDD                   | Test Driven Development                                                                                                                                                                                                                                                                        |
| WP                    | Work Package                                                                                                                                                                                                                                                                                   |

# REFERENCES

**Brennan**, K., 2009, *A guide to the Business analysis body of knowledge
(BABOK guide)*, International Institute of Business Analysis,
Toronto.  
**Devlin**, J., et al., 2019, ‘BERT: Pre-training of Deep
Bidirectional Transformers for Language Understanding’, arXiv
(http://arxiv.org/abs/1810.04805) accessed 28 June 2024.  
**ETC/CE**, et al., 2022, *An overview of Europe’s repair sector*, EIONET Report ETC/CE No
6/2022 (https://www.eionet.europa.eu/etcs/etc-ce/products/etc-ce-products/etc-ce-report-6-2022-an-overview-of-europes-repair-sector).  
**Google Research**, 2024, ‘GitHub - google-research/bert: TensorFlow code and
pre-trained models for BERT’ (https://github.com/google-research/bert)
accessed 28 June 2024.  
**Hruschka**, P. and Starke, G., 2023, arc42 Template
V8.2-EN, (https://arc42.org/download\#file-based-formats).  
**JRC**, 2024, ‘Battery supply chain challenges’, RMIS - Raw Materials Information
System (https://rmis.jrc.ec.europa.eu/analysis-of-supply-chain-challenges-49b749)
accessed 29 June 2024.  
**Martin**, R. C. and Martin, R. C., 2018, *Clean
architecture: a craftsman’s guide to software structure and design*,
Prentice Hall, London, England.  
**Qdrant**, 2024, ‘Qdrant - Vector Database’ (https://qdrant.tech/) accessed 28 June 2024.  
**Salimi**, S., 2024, ‘Persona’, Agile Academy (https://www.agile-academy.com/en/agile-dictionary/persona/) accessed 28
June 2024.  
**Vlachos**, I. and Damvakeraki, T., 2024, *Digital Product
Passport, a Blockchain-based Perspective*, EU Blockchain Observatory and
Forum, Directorate-General for Communications Networks, Content and
Technology, European Commission (https://blockchain-observatory.ec.europa.eu/publications/digital-product-passport-blockchain-based-perspective\_en).

# Footnotes

[1] RE-strategies are strategic actions to enhance the circular economy
of a product. 9 RE-strategies can be defined, 6 of which are within the
project scope: **re**use, **re**pair, **re**purpose, **re**furbish,
**re**manufacture, **re**cycle.

[2] Requirements are divided in Must, Should, Could and Won’t, see e.g.
section 6.1.5 in (Brennan, 2009)

[3] <https://arc42.org/>

[4] All parts and versions of this document have been written with the
help of generative artificial intelligence tools, such as ChatGPT, an
artificial intelligence language model developed by OpenAI. While these
tools assisted in generating content, syntax checking, and typos
verification, all versions have been reviewed and edited by the authors
for accuracy, relevance, and clarity. The responsibility for the content
rests solely with the authors.

[5] <https://semver.org/>

[6] <https://swagger.io/tools/swagger-ui/>

[7] <https://github.com/Redocly/redoc>
